-- functions
function CleanUI_OnLoad(self)
    -- never show the old ui
    UIParent:SetAlpha(0.0);
    CleanUI_StartDelay(CUI_START_DELAY, function() UIParent:SetAlpha(1) end);

    -- max. Kameradistanz erhöhen
    SetCVar("CameraDistanceMax", 50);

    UIParent:SetAttribute("LEFT_OFFSET", 150);
    UIParent:SetAttribute("TOP_OFFSET", -135);

    -- register for addon messages
    RegisterAddonMessagePrefix("CleanUI_XP"); -- XP Bars
    RegisterAddonMessagePrefix("CleanUI_WP"); -- WayPoints

    self:RegisterEvent("ADDON_LOADED");
    self:RegisterEvent("ACHIEVEMENT_EARNED");
    self:RegisterEvent("PLAYER_LEVEL_UP");
    self:RegisterEvent("CVAR_UPDATE");

    --self:RegisterAllEvents();
    
    -- hook methods
    hooksecurefunc("PanelTemplates_SelectTab", CleanUI_PanelTemplates_SelectTab);
    hooksecurefunc("UpdateContainerFrameAnchors", CleanUI_UpdateContainerFrameAnchors);

    -- chat commands
    CleanUI_InitSlashCommandHandler();

    -- optional dependencies
    CleanUI_InitTitanDependency();

    -- fix for 'forbidden' message popup
    hooksecurefunc("StaticPopup_Show", function(which)
        if(which == "ADDON_ACTION_FORBIDDEN") then
            StaticPopup_Hide(which);
        end
    end);
end

function CleanUI_PanelTemplates_SelectTab(tab)
    -- Text nicht nach unten verschieben
    local name = tab:GetName();
    _G[name.."Text"]:SetPoint("CENTER", tab, "CENTER", (tab.deselectedTextX or 0), (tab.deselectedTextY or 2));
end

function CleanUI_UpdateContainerFrameAnchors()
    -- Frame neu positionieren
    DurabilityFrame:ClearAllPoints();
    DurabilityFrame:SetPoint("TOP", UIParent, "TOP", 10, -10);
end

function CleanUI_InitTitanDependency()
    -- titan panel support
    if (TitanUtils_AddonAdjust) then
        TitanUtils_AddonAdjust("Minimap", true);
        TitanUtils_AddonAdjust("MinimapCluster", true);
        TitanUtils_AddonAdjust("PlayerFrame", true);
        TitanUtils_AddonAdjust("TargetFrame", true);
        TitanUtils_AddonAdjust("FocusFrame", true);
    end

    if (TitanPanel_AdjustFrames) then
        hooksecurefunc("TitanPanel_AdjustFrames", CleanUI_TitanPanel_AdjustFrames);
    end
end

function CleanUI_TitanPanel_AdjustFrames()
    CleanUI_UpdateMinimap();
end

function CleanUI_GetTitanYOffset()
    if (TitanMovable_GetPanelYOffset) then
        return TitanMovable_GetPanelYOffset(TITAN_PANEL_PLACE_TOP);
    end

    return 0;
end

function CleanUI_InitSlashCommandHandler()

    SLASH_CLEANUI1 = "/cui";
    SLASH_CLEANUI2 = "/cleanui";
    SlashCmdList["CLEANUI"] = function(msg)
        CleanUI_SlashCommandHandler(msg);
    end
end

function CleanUI_SlashCommandHandler(msg)
    if (msg == "reload") then
        ReloadUI();
    elseif (msg == "gc") then
        CleanUI_GC();
    elseif (msg == "build") then
        CleanUI_WoWBuildInfo();
    elseif (msg == "versionupdate" or msg == "vu") then
        CleanUIData.VERSION = nil;
        CleanUIConfig_VersionUpdate();
    elseif (msg == "reset") then
        CleanUIConfig_Reset();
    elseif (string.find(msg, "framestack") == 1) then
        GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
        GameTooltip:SetFrameStack(true);
    elseif (string.find(msg, "wp") == 1) then
        local coords = string.sub(msg, 4);
        CleanUI_AddWaypoint(coords);
    elseif (string.find(msg, "fake") == 1) then
        local message = string.sub(msg, 6);
        CleanUI_XXXXX(message);
    end

    CleanUIConfig_ReloadConfig();
end

function CleanUI_OnEvent(self, event, ...)

    --cui_debug(event, "event");

    if (event == "ADDON_LOADED") then
        local addonName = ...;

        if (addonName == "CleanUI") then
            CleanUI_StartDelay(CUI_START_DELAY, CleanUI_InitCommon);
            CleanUI_StartDelay(CUI_START_DELAY, function (self) CleanUI_ReplaceAllBorderTextures("UI-Tooltip-Border", UIParent); end);
        end
    end

    if (event == "ACHIEVEMENT_EARNED" or event == "PLAYER_LEVEL_UP") then
        if (CleanUIData.SCREENSHOT) then
            CleanUI_TakeScreenshot();
        end
    end

    if (event == "CVAR_UPDATE") then
    local cvar, value = ...;
    cui_debug(value, cvar);
    end
end

function CleanUI_InitCommon()

    CleanUIConfig_Init();

    DropDownList1Backdrop:SetBackdrop(nil);
    DropDownList2Backdrop:SetBackdrop(nil);
    DropDownList1MenuBackdrop:SetBackdrop(nil);
    DropDownList2MenuBackdrop:SetBackdrop(nil);

    CleanUI_SetBackdrop(DropDownList1Backdrop);
    CleanUI_SetBackdrop(DropDownList1MenuBackdrop);
    CleanUI_SetBackdrop(DropDownList2Backdrop);
    CleanUI_SetBackdrop(DropDownList2MenuBackdrop);

    if (CleanUIData.TOOLTIPS_BACKDROP) then
        CleanUI_SetBackdrop(GameTooltip);
        CleanUI_SetBackdrop(ShoppingTooltip1);
        CleanUI_SetBackdrop(ShoppingTooltip2);

        CleanUI_SetBackdrop(SmallTextTooltip);
    end

    CleanUI_SetBackdrop(ItemRefTooltip);
    CleanUI_SetBackdrop(ItemRefShoppingTooltip1);
    CleanUI_SetBackdrop(ItemRefShoppingTooltip2);

    CleanUI_SetBackdrop(WorldMapTooltip);
    CleanUI_SetBackdrop(WorldMapCompareTooltip1);
    CleanUI_SetBackdrop(WorldMapCompareTooltip2);

    CleanUI_MakeFrameMoveable(AddonList);
    CleanUI_MakeFrameMoveable(WorldMapFrame);
    CleanUI_MakeFrameMoveable(CharacterFrame);
    CleanUI_MakeFrameMoveable(FriendsFrame);
    CleanUI_MakeFrameMoveable(SpellBookFrame);
    CleanUI_MakeFrameMoveable(LFDParentFrame);
    CleanUI_MakeFrameMoveable(MerchantFrame);

    CleanUI_InitComponents();
    
    CleanUI_StartDelay(1.0, CleanUI_KillTutorials);

    CleanUI_InitStatusBarTexture();
    CleanUI_UpdateRole();
    CleanUI_UpdateContainerFrameAnchors();
end

function CleanUI_KillTutorials()
    SetCVar("lastGarrisonMissionTutorial", 8);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_GARRISON_LANDING, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_GARRISON_BUILDING, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_GARRISON_ZONE_ABILITY, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_CLEAN_UP_BAGS, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TOYBOX_FAVORITE, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TOYBOX_MOUSEWHEEL_PAGING, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_REAGENT_BANK_UNLOCK, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_GLYPH, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_TALENT, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_SPEC, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_BAG_SETTINGS, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_PET_JOURNAL, true);
    SetCVarBitfield("closedInfoFrames", LE_FRAME_TUTORIAL_LFG_LIST, true);
end

function CleanUI_InitComponents()
    local delay = 0.1;

    CleanUI_StartDelay(delay, CleanUI_InitDataStore);

    CleanUI_StartDelay(delay, CleanUI_InitAddonList);
    CleanUI_StartDelay(delay, CleanUI_InitActionBars);
    CleanUI_StartDelay(delay, CleanUI_InitPopups);
    CleanUI_StartDelay(delay, CleanUI_InitBags);
    CleanUI_StartDelay(delay, CleanUI_InitBattlePets);
    CleanUI_StartDelay(delay, CleanUI_InitCastingBars);
    CleanUI_StartDelay(delay, CleanUI_InitCharacterFrame);
    CleanUI_StartDelay(delay, CleanUI_InitChat);
    CleanUI_StartDelay(delay, CleanUI_InitClassTrainerFrame);
    CleanUI_StartDelay(delay, CleanUI_InitDataBroker);
    CleanUI_StartDelay(delay, CleanUI_InitDeBuffs);
    CleanUI_StartDelay(delay, CleanUI_InitEquipmentSets);
    CleanUI_StartDelay(delay, CleanUI_InitGuild);
    CleanUI_StartDelay(delay, CleanUI_InitMail);
    CleanUI_StartDelay(delay, CleanUI_InitMap);
    CleanUI_StartDelay(delay, CleanUI_InitMinimap);
    CleanUI_StartDelay(delay, CleanUI_InitNameplates);
    CleanUI_StartDelay(delay, CleanUI_InitQuestFrame);
    CleanUI_StartDelay(delay, CleanUI_InitGossipFrame);
    CleanUI_StartDelay(delay, CleanUI_InitGarrisonFrame);
    CleanUI_StartDelay(delay, CleanUI_InitRaid);
    CleanUI_StartDelay(delay, CleanUI_InitReputation);
    CleanUI_StartDelay(delay, CleanUI_InitRoleCheck);
    CleanUI_StartDelay(delay, CleanUI_InitTalents);
    CleanUI_StartDelay(delay, CleanUI_InitTooltips);
    CleanUI_StartDelay(delay, CleanUI_InitUnitFrames);
    CleanUI_StartDelay(delay, CleanUI_InitXpFrame);
    CleanUI_StartDelay(delay, CleanUI_InitTradeSkillFrame);
    CleanUI_StartDelay(delay, CleanUI_InitDungeonBrowser);
    CleanUI_StartDelay(delay, CleanUI_InitMerchantFrame);
    CleanUI_StartDelay(delay, CleanUI_InitTalentFrame);
    CleanUI_StartDelay(delay, CleanUI_InitTradeFrame);
    CleanUI_StartDelay(delay, CleanUI_InitAuctionFrame);
    CleanUI_StartDelay(delay, CleanUI_InitSpellBookFrame);
    CleanUI_StartDelay(delay, CleanUI_InitPetJournal);
    CleanUI_StartDelay(delay, CleanUI_InitPvpFrame);
    CleanUI_StartDelay(delay, CleanUI_InitFriendsFrame);
    CleanUI_StartDelay(delay, CleanUI_InitCalendarFrame);
    CleanUI_StartDelay(delay, CleanUI_InitMacroFrame);
    CleanUI_StartDelay(delay, CleanUI_InitReforgingFrame);
    CleanUI_StartDelay(delay, CleanUI_InitTransmogrifyFrame);
    CleanUI_StartDelay(delay, CleanUI_InitItemUpgradeFrame);
    
    CleanUI_StartDelay(delay, CleanUI_InitTopBar);
    
    CleanUI_StartDelay(delay, CleanUI_InitSelfieCam);
    
    CleanUI_StartDelay(delay, CleanUI_InitDataStoreUI);

    -- other addons
    CleanUI_StartDelay(delay, CleanUI_InitExtAddons);
end

function CleanUI_InitStatusBarTexture()
    -- unit frames
    PlayerFrameHealthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    PlayerFrameManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    PlayerFrameAlternateManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    PlayerFrameMyHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    PlayerFrameOtherHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    PlayerFrameTotalAbsorbBar:SetTexture(CleanUIData.CUI_STATUSBAR);

    PetFrameHealthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    PetFrameManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    PetFrameMyHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    PetFrameOtherHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    PetFrameTotalAbsorbBar:SetTexture(CleanUIData.CUI_STATUSBAR);

    TargetFrameHealthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    TargetFrameManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    TargetFrameMyHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    TargetFrameOtherHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    TargetFrameTotalAbsorbBar:SetTexture(CleanUIData.CUI_STATUSBAR);

    FocusFrameHealthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    FocusFrameManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    FocusFrameMyHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    FocusFrameOtherHealPredictionBar:SetTexture(CleanUIData.CUI_STATUSBAR);
    FocusFrameTotalAbsorbBar:SetTexture(CleanUIData.CUI_STATUSBAR);

    TargetFrameToTHealthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    TargetFrameToTManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    FocusFrameToTHealthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    FocusFrameToTManaBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    -- party unit frames
    for i=1, 4, 1 do
        _G["PartyMemberFrame"..i.."HealthBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
        _G["PartyMemberFrame"..i.."ManaBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
        _G["PartyMemberFrame"..i.."PetFrameHealthBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

        _G["PartyMemberFrame"..i.."MyHealPredictionBar"]:SetTexture(CleanUIData.CUI_STATUSBAR);
        _G["PartyMemberFrame"..i.."OtherHealPredictionBar"]:SetTexture(CleanUIData.CUI_STATUSBAR);
        _G["PartyMemberFrame"..i.."TotalAbsorbBar"]:SetTexture(CleanUIData.CUI_STATUSBAR);

        if (_G["CUI_Party"..i.."PetManaBar"]) then
            _G["CUI_Party"..i.."PetManaBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
        end
    end

    CastingBarFrame:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    _G["CleanUIPetCastingBarFrame"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    _G["CleanUICastingBarGlobalCooldownFrame"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    _G["CleanUICastingBarLatency"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    for index=1, MIRRORTIMER_NUMTIMERS do
        _G["MirrorTimer"..index.."StatusBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    end

    -- spell bars
    TargetFrameSpellBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    FocusFrameSpellBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    -- exp bars
    MainMenuExpBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    ReputationWatchStatusBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    -- party exp bars
    for i=1, 4, 1 do
        _G["CleanUIXpFrame_group"..i.."xpbar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
        _G["CleanUIXpFrame_group"..i.."xpbarExhaustion"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    end

    -- boss frames
    for i=1, 5, 1 do
        _G["Boss"..i.."TargetFrameHealthBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
        _G["Boss"..i.."TargetFrameManaBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
        _G["Boss"..i.."TargetFrameSpellBar"]:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
    end

    for rg = 1, 8 do
        local raidGroup = _G["CompactRaidGroup"..rg];

        if (raidGroup) then
            for uu = 1, 5 do
                local unitFrame = _G[raidGroup:GetName().."Member"..uu];

                unitFrame.healthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
                unitFrame.powerBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
            end
        end
    end

    CleanUI_UpdateComboPointTexture(CleanUIData.CUI_STATUSBAR);

    CleanUI_UpdateNameplatesTextures(CleanUIData.CUI_STATUSBAR);

    CleanUI_HookChangeStatusBar(CleanUIData.CUI_STATUSBAR);
end
