local waitTable = {};
local waitFrame = nil;

local debug = true;
-- functions
function cui_debug(value, name)
    if (not debug) then
        return;
    end

    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("debug "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("debug: "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

-- functions
function cui_info(value, name)
    if (name) then
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ffCleanUI:|r "..name..": "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    else
        DEFAULT_CHAT_FRAME:AddMessage("|c333399ffCleanUI:|r "..(value or "nil"), 1.0, 1.0, 1.0, 1, 10);
    end
end

function CleanUI_TakeScreenshot()
    CleanUI_StartDelay(1.2, Screenshot);
end

function CleanUI_RandomPet()
    SummonRandomCritter();
end

function CleanUI_TakeScreenshotWithoutUI()
    if ( UIParent:IsShown() ) then
        securecall("CloseMenus");
        securecall("CloseAllWindows");
        UIParent:Hide();
        SetUIVisibility(false);
    end

    --TakeScreenshot();

    local screenshotFrame = _G["CUI_SCREENSHOT_FRAME"];
    if (not screenshotFrame) then
        screenshotFrame = CreateFrame("Frame", "CUI_SCREENSHOT_FRAME");
    end

    screenshotFrame.last = 0.0;
    screenshotFrame.shot = 1.0;
    screenshotFrame.throttle = 3.0;
    screenshotFrame.foto = nil;

    screenshotFrame:SetScript('OnUpdate', function(self, elapsed)
        self.last = self.last + elapsed;

        if (self.last >= self.shot and not self.foto) then
            TakeScreenshot();
            self.foto = 1;
        end

        if (self.last >= self.throttle) then
            self:SetScript('OnUpdate', nil);
            UIParent:Show();
            SetUIVisibility(true);
        end
    end)

    screenshotFrame:Show();
end

function CleanUI_StartDelay(delay, func, ...)
    if(type(delay)~="number" or type(func)~="function") then
        return false;
    end

    if(waitFrame == nil) then
        waitFrame = CreateFrame("Frame","CUI_WaitFrame", UIParent);
        waitFrame:SetScript("onUpdate",function (self,elapse)
            local count = #waitTable;
            local i = 1;

            while(i<=count) do
                local waitRecord = tremove(waitTable,i);
                local d = tremove(waitRecord,1);
                local f = tremove(waitRecord,1);
                local p = tremove(waitRecord,1);

                if(d>elapse) then
                    tinsert(waitTable,i,{d-elapse,f,p});
                    i = i + 1;
                else
                    count = count - 1;
                    f(unpack(p));
                end
            end
        end);
    end

    tinsert(waitTable,{delay,func,{...}});

    return true;
end

function CleanUI_MakeFrameMoveable(frame)
    frame:SetMovable(true);
    frame:SetUserPlaced(true);
    frame:SetClampedToScreen(true);
    frame:SetClampRectInsets(1, -1, -1, 1);

    frame:HookScript("OnMouseDown", CleanUI_AnyFrameOnMouseDown);
    frame:HookScript("OnMouseUp", CleanUI_AnyFrameOnMouseUp);
end

function CleanUI_AnyFrameOnMouseDown(self, button)
    if (button == "LeftButton") then
        self:StartMoving();
    end
end

function CleanUI_AnyFrameOnMouseUp(self, button)
    self:StopMovingOrSizing();
end

function cui_rgb2hex(red, green, blue)

    if (not red) then
        red = 1.0;
    end

    if (not green) then
        green = 1.0;
    end

    if (not blue) then
        blue = 1.0;
    end

    local r = string.format("%x", red * 255);
    local g = string.format("%x", green * 255);
    local b = string.format("%x", blue * 255);

    if (strlen(r) < 2) then
        r = "0"..r;
    end
    if (strlen(g) < 2) then
        g = "0"..g;
    end
    if (strlen(b) < 2) then
        b = "0"..b;
    end

    local color = r..g..b;

    return color;
end

function CleanUI_ReplaceAllTextures(texturename, frame, withTexture)

    if (not CleanUIData.AUTO_CHANGE_BORDER_STYLE) then
        return;
    end

    for i, child in ipairs({frame:GetChildren()}) do

        if (child and not child:IsForbidden() and child:IsObjectType("Frame")) then
            CleanUI_ReplaceTextureWith(texturename, child, withTexture);

            if (child:IsObjectType("frame")) then
                CleanUI_ReplaceAllTextures(texturename, child, withTexture);
            end
        end
    end
end

function CleanUI_ReplaceTextureWith(texturename, frame, withTexture)
    for i, child in ipairs({frame:GetRegions()}) do
    
        if (child and not child:IsForbidden() and child:IsObjectType("texture")) then
            local texture = child:GetTexture();
            if (texture) then
                if (string.find(texture, texturename, 1, true)) then
                    child:SetTexture(withTexture);
                end
            end
        end
    end
end

function CleanUI_RemoveTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do

        if (child and child:IsObjectType("texture")) then
            child:SetTexture(nil);
        end
    end
end

function CleanUI_GetRegionOfTypeAtPos(frame, type, pos)
    local index = 0;
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsShown() and child:IsObjectType(type)) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_GetChildOfTypeAtPos(frame, type, pos)
    local index = 0;
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsShown() and child:IsObjectType(type)) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_GetChildWithName(frame, name)
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:GetName() == name) then
            return child;
        end
    end
end

function CleanUI_GetChildWithNameAtPos(frame, name, pos)
    local index = 0;
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:GetName() == name) then
            if (index == pos) then
                return child;
            end
            index = index + 1;
        end
    end
end

function CleanUI_ListChildren(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child) then
            cui_debug(child:GetName(), "region");
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            cui_debug(child:GetName(), "frame");

            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_ShowChildren(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child) then
            cui_debug(child:GetName(), "region");
            child:Show();
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            cui_debug(child:GetName(), "frame");
            child:Show();

            if (not child:IsObjectType("button")) then
                CleanUI_ShowTextures(child);
            end
        end
    end
end

function CleanUI_ListTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsObjectType("texture")) then
            local texture = child:GetTexture();
            if (texture) then
                cui_debug(texture, child:GetName());
            end
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_ShowTextures(frame)
    for i, child in ipairs({frame:GetRegions()}) do
        if (child and child:IsObjectType("texture")) then
            child:Show();
            local texture = child:GetTexture();
            if (texture) then
                cui_debug(texture, child:GetName());
            end
        end
    end
    for i, child in ipairs({frame:GetChildren()}) do
        if (child and child:IsObjectType("Frame")) then
            child:Show();
            if (not child:IsObjectType("button")) then
                CleanUI_ListTextures(child);
            end
        end
    end
end

function CleanUI_GC()
    local before = collectgarbage("count");
    collectgarbage();
    local after = collectgarbage("count");

    local freed = before - after;

    local msg = "collectgarbage:\n"..
        "memory freed: |c00FFFF66"..string.format("%.2f", freed).." kb|r.\n"..
        "current memory use: |c00FFFF66"..string.format("%.2f", after).." kb|r.\n";

    DEFAULT_CHAT_FRAME:AddMessage(msg, 1.0, 1.0, 1.0, 1, 10);
end

function CleanUI_WoWBuildInfo()
    local version, build, date, tocversion = GetBuildInfo();
    cui_debug(version, "version");
    cui_debug(build, "build");
    cui_debug(date, "date");
    cui_debug(tocversion, "tocversion");
end

-- hook
function CleanUI_HookChangeStatusBar(texture)
end

function CleanUI_XXXXX(message)
    cui_debug(message, "message");
end
