local insetValue = 2;

-- Rahmen
CUI_FRAME_BORDER = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-border";
CUI_FRAME_BORDER_STANCEBAR = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-border-stancebar";

-- Hintergrund
CUI_FRAME_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background";
CUI_FRAME_BACKGROUND1 = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background1";
CUI_FRAME_BACKGROUND2 = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background2";
CUI_FRAME_BACKGROUND_OPAQUE = "Interface\\AddOns\\CleanUI\\skins\\frame\\frame-background-opaque";
CUI_TAB_BACKGROUND = "Interface\\AddOns\\CleanUI\\skins\\frame\\tab-background";
CUI_TAB_BACKGROUND_RIGHT = "Interface\\AddOns\\CleanUI\\skins\\frame\\tab-background-right";

KEY_NUMPAD0 = "N0";
KEY_NUMPAD1 = "N1";
KEY_NUMPAD2 = "N2";
KEY_NUMPAD3 = "N3";
KEY_NUMPAD4 = "N4";
KEY_NUMPAD5 = "N5";
KEY_NUMPAD6 = "N6";
KEY_NUMPAD7 = "N7";
KEY_NUMPAD8 = "N8";
KEY_NUMPAD9 = "N9";
KEY_NUMPADDECIMAL = "N,";
KEY_NUMPADDIVIDE = "N/";
KEY_NUMPADMINUS = "N-";
KEY_NUMPADMULTIPLY = "N*";
KEY_NUMPADPLUS = "N+";
KEY_NUMPADEQUALS = "N=";

function CleanUI_SetBackdrop(frame, addBackground, opaque, otherborder, otherbg, top, left, bottom, right)
    local border = otherborder or CleanUI_GetFrameBorder();
    local bg = otherbg or CUI_FRAME_BACKGROUND;

    local parent = frame;

    if (top or left or bottom or right) then
        local backdropFrame = _G[frame:GetName().."BackdropFrame"];
        if (not backdropFrame) then
            backdropFrame = CreateFrame("Frame", frame:GetName().."BackdropFrame", frame);
            backdropFrame:SetParent(frame);
            backdropFrame:SetFrameStrata("BACKGROUND");
        end

        backdropFrame:ClearAllPoints();
        backdropFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", (left or 0), (top or 0));
        backdropFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", (right or 0), (bottom or 0));

        parent = backdropFrame;
    end

    if (opaque) then
        bg = CUI_FRAME_BACKGROUND_OPAQUE;
    end

    if (addBackground == false) then
        parent:SetBackdrop( {
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    else
        parent:SetBackdrop( {
            bgFile = bg,
            edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
            insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
        });
    end
end

function CleanUI_SetBackdropWithoutBorder(frame, opaque)
    local bg = CUI_FRAME_BACKGROUND;

    if (opaque) then
        bg = CUI_FRAME_BACKGROUND_OPAQUE;
    end

    frame:SetBackdrop( {
        bgFile = bg, tile = true, tileSize = 64, edgeSize = 16,
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });
end

function CleanUI_SetGraphicsBackground(frame, title, closeButton, top, left, bottom, right)
    top = 20 + (top or 0);
    left = -15 + (left or 0);
    bottom = -20 + (bottom or 0);
    right = 15 + (right or 0);
    
    local background = CreateFrame("Frame", frame:GetName().."TexturedBackground", frame, "CUITexturedBackgroundTemplate");
    background:SetParent(frame);
    background:SetPoint("TOPLEFT", frame, "TOPLEFT", left, top);
    background:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", right, bottom);
    
    background:SetFrameStrata("LOW");
    
    if (closeButton) then
        closeButton:ClearAllPoints();
        closeButton:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -8 + right, -8 + top);
    end
    
    if (title) then
        background.HeaderBar:ClearAllPoints();
        background.HeaderBar:SetPoint("TOPLEFT", title, "TOPLEFT", 0, 8);
        background.HeaderBar:SetPoint("BOTTOMRIGHT", title, "BOTTOMRIGHT", 0, -8);
    
    else
        background.HeaderBar:Hide();
    end
end

function CleanUI_GetFrameBorder()
    return CUI_FRAME_BORDER;
end

function CleanUI_GetFrameBackground()
    return CUI_FRAME_BACKGROUND;
end

function CleanUI_ReplaceAllBorderTextures(texturename, frame)
    CleanUI_ReplaceAllTextures(texturename, frame, CleanUI_GetFrameBorder());
end

function CleanUI_CreateHeaderFrame(parent, name, width, titleTextField, titleText, red, green, blue)
    local fontString, objectType;

    local borderframe = _G[name];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", name, parent);

        CleanUI_SetBackdrop(borderframe, true, true);

        borderframe:SetSize(width, 28);

        borderframe:ClearAllPoints();
        borderframe:SetPoint("CENTER", parent, "TOP", 0, -3);
    end

    if (titleTextField) then
        fontString = titleTextField;
        if (titleText) then
            fontString:SetText(titleText);
        end
    else
        fontString = borderframe:CreateFontString(name.."FontString");
        fontString:SetFontObject("SystemFont_Med3"); -- MailTextFontNormal
        fontString:SetWidth(width);
        fontString:SetHeight(25);
        fontString:SetText(titleText);
    end

    fontString:SetFontObject("SystemFont_Med3"); -- MailTextFontNormal

    fontString:SetParent(borderframe);
    fontString:ClearAllPoints();
    fontString:SetAllPoints(borderframe);
    fontString:SetJustifyH("CENTER");

    fontString:SetTextColor(red or 0.3, green or 0.7, blue or 1.0);
    
    borderframe:SetFrameStrata("HIGH");

    return borderframe, fontString;
end

function CleanUI_ChangeCloseButton(frame, buttonname, closeButton)
    if (not closeButton) then
        if (not buttonname) then
            buttonname = frame:GetName().."CloseButton";
        end

        closeButton = _G[buttonname];
    end

    closeButton:ClearAllPoints();
    closeButton:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -1, -1);

    closeButton:SetSize(22, 22);

    closeButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\closebutton");
    closeButton:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\closebutton-pressed");
    closeButton:SetHighlightTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\closebutton-hover");
    closeButton:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\closebutton-disabled");

    closeButton:GetHighlightTexture():SetBlendMode("BLEND");
end

function CleanUI_ChangeNextButton(button)
    CleanUI_RemoveTextures(button);
    button:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Up");
    button:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-NextPage-Down");
    button:SetDisabledTexture(nil);
    button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight");
end

function CleanUI_ChangePrevButton(button)
    CleanUI_RemoveTextures(button);
    button:SetNormalTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Up");
    button:SetPushedTexture("Interface\\Buttons\\UI-SpellbookIcon-PrevPage-Down");
    button:SetDisabledTexture(nil);
    button:SetHighlightTexture("Interface\\Buttons\\UI-Common-MouseHilight");
end

function CleanUI_ChangeButton(button)
    CleanUI_ChangeButtonWithColor(button, "blue");
end

function CleanUI_ChangeButtonRed(button)
    CleanUI_ChangeButtonWithColor(button, "red");
end

function CleanUI_ChangeButtonWithColor(button, color)
    if (not button) then
        return;
    end

    CleanUI_RemoveTextures(button);
    --CleanUI_SetBackdrop(button, true, true);

    local name = button:GetName();

    if (name) then
        local background_left = _G[name.."BackgroundLeft"];
        if (not background_left) then
            background_left = button:CreateTexture(name.."BackgroundLeft", "BACKGROUND");
            background_left:SetSize(16, 32);
            background_left:SetPoint("TOPLEFT", button, "TOPLEFT", 0, 0);
            background_left:SetPoint("BOTTOMRIGHT", button, "BOTTOMLEFT", 16, 0);
            background_left:SetTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\button-left");
        end

        local background_right = _G[name.."BackgroundRight"];
        if (not background_right) then
            background_right = button:CreateTexture(name.."BackgroundRight", "BACKGROUND");
            background_right:SetSize(16, 32);
            background_right:SetPoint("TOPLEFT", button, "TOPRIGHT", -16, 0);
            background_right:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", 0, 0);
            background_right:SetTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\button-right");
        end

        local background_middle = _G[name.."BackgroundMiddle"];
        if (not background_middle) then
            background_middle = button:CreateTexture(name.."BackgroundMiddle", "BACKGROUND");
            background_middle:SetSize(32, 32);
            background_middle:SetPoint("TOPLEFT", background_left, "TOPRIGHT", 0, 0);
            background_middle:SetPoint("BOTTOMRIGHT", background_right, "BOTTOMLEFT", 0, 0);
            background_middle:SetTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\button-middle");
        end
    end

    button:SetHighlightTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\button-highlight-"..color);

    button:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);
    button:GetHighlightTexture():ClearAllPoints();
    button:GetHighlightTexture():SetPoint("TOPLEFT", button, "TOPLEFT", 4, -2);
    button:GetHighlightTexture():SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -4, 2);
    button:GetHighlightTexture():SetBlendMode("MOD");
    button:SetDisabledTexture(nil);

    button:HookScript("OnLoad", CleanUI_RemoveButtonTextures);
    button:HookScript("OnMouseDown", CleanUI_RemoveButtonTextures);
    button:HookScript("OnMouseUp", CleanUI_RemoveButtonTextures);
    button:HookScript("OnShow", CleanUI_RemoveButtonTextures);
    button:HookScript("OnDisable", CleanUI_RemoveButtonTextures);
    button:HookScript("OnEnable", CleanUI_RemoveButtonTextures);
end

function CleanUI_ChangeFilterButton(button, addBackdrop, top, left, bottom, right)
    CleanUI_RemoveTextures(button);
    button:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight");

    top = top or 0;
    left = left or 0;
    bottom = bottom or 0;
    right = right or 0;

    if (addBackdrop) then
        local name = button:GetName();

        if (name) then
            local background_left = _G[name.."BackgroundLeft"];
            if (not background_left) then
                background_left = button:CreateTexture(name.."BackgroundLeft", "BACKGROUND");
                background_left:SetSize(16, 32);
                background_left:SetPoint("TOPLEFT", button, "TOPLEFT", left, top);
                background_left:SetPoint("BOTTOMRIGHT", button, "BOTTOMLEFT", 16, bottom);
                background_left:SetTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\filterbutton-left");
            end

            local background_right = _G[name.."BackgroundRight"];
            if (not background_right) then
                background_right = button:CreateTexture(name.."BackgroundRight", "BACKGROUND");
                background_right:SetSize(16, 32);
                background_right:SetPoint("TOPLEFT", button, "TOPRIGHT", -16, top);
                background_right:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", right, bottom);
                background_right:SetTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\filterbutton-right");
            end

            local background_middle = _G[name.."BackgroundMiddle"];
            if (not background_middle) then
                background_middle = button:CreateTexture(name.."BackgroundMiddle", "BACKGROUND");
                background_middle:SetSize(32, 32);
                background_middle:SetPoint("TOPLEFT", background_left, "TOPRIGHT", 0, 0);
                background_middle:SetPoint("BOTTOMRIGHT", background_right, "BOTTOMLEFT", 0, 0);
                background_middle:SetTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\filterbutton-middle");
            end
        end
    end
end

function CleanUI_ChangeNextButton(button)
    button:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\colapse");
    button:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\colapse_down");
    button:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\empty");
end

function CleanUI_ChangeBackButton(button)
    button:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\expand");
    button:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\databroker\\expand_down");
    button:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\empty");
end

function CleanUI_ChangeDropDown(dropdown)
    CleanUI_RemoveTextures(dropdown);

    local button = _G[dropdown:GetName().."Button"];

    if (button) then
        button:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\dropdown");
        button:SetPushedTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\dropdown-pressed");
        button:SetHighlightTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\dropdown-hover");
        button:SetDisabledTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\dropdown-disabled");
    end
end

function CleanUI_RemoveButtonTextures(self)
    if (self.Left) then
        self.Left:SetTexture(nil);
    end

    if (self.Middle) then
        self.Middle:SetTexture(nil);
    end

    if (self.Right) then
        self.Right:SetTexture(nil);
    end
end

function CleanUI_ChangeEditBox(self)
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border");
    CleanUI_ReplaceTextureWith("Interface\\ClassTrainerFrame\\UI-ClassTrainer-FilterBorder", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\UI-ClassTrainer-FilterBorder");
    
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-B", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-B");
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-BL", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-BL");
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-BR", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-BR");
    
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-L", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-L");
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-M", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-M");
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-R", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-R");
    
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-T", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-T");
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-TL", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-TL");
    CleanUI_ReplaceTextureWith("Interface\\Common\\Common-Input-Border-TR", self, "Interface\\AddOns\\CleanUI\\skins\\editbox\\Common-Input-Border-TR");
end

function CleanUI_ChangeTab(tabButton, offset)
    CleanUI_RemoveTextures(tabButton);

    local borderframe = _G[tabButton:GetName().."Borderframe"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", tabButton:GetName().."Borderframe", tabButton);
    end

    local top = 3;
    if (offset) then
        top = top + offset;
    end

    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", tabButton, "TOPLEFT", 8, top);
    borderframe:SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", -8, 3);

    borderframe:SetFrameStrata("LOW");

    borderframe:SetBackdrop( {
        bgFile = CUI_TAB_BACKGROUND,
        edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });

    tabButton:SetHighlightTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\button-highlight-blue");
    tabButton:GetHighlightTexture():SetTexCoord(0, 1, 0, 1);
    tabButton:GetHighlightTexture():ClearAllPoints();
    tabButton:GetHighlightTexture():SetPoint("TOPLEFT", borderframe, "TOPLEFT", 1, -3);
    tabButton:GetHighlightTexture():SetPoint("BOTTOMRIGHT", borderframe, "BOTTOMRIGHT", -2, 2);
    tabButton:GetHighlightTexture():SetBlendMode("MOD");
end

function CleanUI_ChangeSideTab(tabButton)
    CleanUI_RemoveTextures(tabButton);

    local borderframe = _G[tabButton:GetName().."Borderframe"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", tabButton:GetName().."Borderframe", tabButton);
    end

    borderframe:ClearAllPoints();
    borderframe:SetPoint("TOPLEFT", tabButton, "TOPLEFT", -4, -3);
    borderframe:SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 0, 3);

    borderframe:SetFrameStrata("LOW");

    borderframe:SetBackdrop( {
        bgFile = CUI_TAB_BACKGROUND_RIGHT,
        edgeFile = border, tile = true, tileSize = 64, edgeSize = 16,
        insets = { left = insetValue, right = insetValue, top = insetValue, bottom = insetValue }
    });

    local button = _G[tabButton:GetName().."Button"];
    button:SetNormalTexture(nil);
    button:SetPushedTexture(nil);
    button:SetHighlightTexture(nil);

    local checkedTexture = button:GetCheckedTexture();
    checkedTexture:ClearAllPoints();
    checkedTexture:SetPoint("TOPLEFT", tabButton, "TOPLEFT", -3, -3);
    checkedTexture:SetPoint("BOTTOMRIGHT", tabButton, "BOTTOMRIGHT", 0, 3);
    button:SetCheckedTexture("Interface\\AddOns\\CleanUI\\skins\\highlight");

    local icon = _G[tabButton:GetName().."ButtonIconTexture"];
    icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
end
