local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

function CleanUI_InitActionBars()
    if (not CleanUIData.CHANGE_ACTIONBARS) then
        return;
    end

    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI", "Action Buttons");
    end

    OverrideActionBarExpBar:SetFrameStrata("BACKGROUND");
    OverrideActionBarExpBarXpMid:SetDrawLayer("OVERLAY");
    OverrideActionBarExpBarXpL:SetDrawLayer("OVERLAY");
    OverrideActionBarExpBarXpR:SetDrawLayer("OVERLAY");

    CleanUI_SetBackdrop(CleanUIActionButtonsArtFrame, true);
    CleanUI_SetBackdrop(CleanUIStanceBarArtFrame, true, false, CUI_FRAME_BORDER_STANCEBAR);
    CleanUI_SetBackdrop(CleanUIPetActionButtonBarArtFrame, true);

    CleanUI_MainMenuBar_UpdateArt();
    CleanUI_MultiActionBar_Update();
    CleanUI_CleanActionButtons();
    CleanUI_RemoveArtwork();
    CleanUI_ChangeActionBarUpDownButtons();
    CleanUI_ChangeMicroButtons();

    CleanUI_StanceBar_UpdateState();
    CleanUI_PetActionBar_Update();

    CleanUI_ActionBarsHookEvents();
    CleanUI_RegisterActionBarEvents();

    CleanUI_ReorderActionButtons();
    CleanUI_ReorderExpBar();
    CleanUI_ManageFramePositions();

    CleanUI_RemoveActionButtonAnimations();

    PetActionBar_UpdatePositionValues();
    PetActionBarFrame:SetPoint("TOPLEFT", MainMenuBar, "BOTTOMLEFT", PETACTIONBAR_XPOS, PETACTIONBAR_YPOS);

    -- ComboPoints
    MainMenuBar.combopoints = CreateFrame("Frame", "CUIMainMenuBarComboPoints", MainMenuBar);
    MainMenuBar.combopoints:SetSize(300, 14);
    MainMenuBar.combopoints:ClearAllPoints();
    MainMenuBar.combopoints:SetPoint("BOTTOM", CleanUIActionButtonsArtFrame, "TOP", 0, -4);

    CleanUI_InitActionBarCombopoints(MainMenuBar.combopoints);

    MainMenuBar.combopoints:Hide();
end

function CleanUI_InitActionBarCombopoints(parent)
    local last = nil;
    for i=1, 5 do
        local combo = _G[parent:GetName()..i];
        if (combo == nil) then
            combo = parent:CreateTexture(parent:GetName()..i, "OVERLAY");
            combo:SetWidth(60);
            combo:SetHeight(12);
            combo:SetTexture(CleanUIData.CUI_STATUSBAR);

            if (i == 1) then
                combo:SetVertexColor(0.25, 0.75, 0.25, 1.0);
            elseif (i == 2) then
                combo:SetVertexColor(1.00, 1.00, 0.00, 1.0);
            elseif (i == 3) then
                combo:SetVertexColor(1.00, 0.75, 0.25, 1.0);
            elseif (i == 4) then
                combo:SetVertexColor(1.00, 0.50, 0.25, 1.0);
            elseif (i == 5) then
                combo:SetVertexColor(1.00, 0.10, 0.10, 1.0);
            end

            combo:Hide();
        end

        if (last) then
            combo:ClearAllPoints();
            combo:SetPoint("LEFT", last, "RIGHT", 1, 0);
        else
            combo:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0);
        end

        last = combo;
    end
end

function CleanUI_RemoveActionButtonAnimations()
    CleanUI_RemoveActionButtonAnimationForBar(MainMenuBar);
    CleanUI_RemoveActionButtonAnimationForBar(MultiBarRight);
    CleanUI_RemoveActionButtonAnimationForBar(OverrideActionBar);
end

function CleanUI_RemoveActionButtonAnimationForBar(bar)
    local animation = bar.slideOut:GetAnimations();
    animation:SetDuration(0.01);
end

function CleanUI_RegisterActionBarEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("PET_BATTLE_OPENING_START");
    frame:RegisterEvent("PET_BATTLE_CLOSE");
    frame:RegisterEvent("UNIT_ENTERED_VEHICLE");
    frame:RegisterEvent("UNIT_EXITED_VEHICLE");

    frame:HookScript("OnEvent", CleanUI_ActionBarsOnEvent);
end

function CleanUI_ActionBarsOnEvent(self, event, ...)
    local arg1 = ...;

    -- pet battles
    if (event == "PET_BATTLE_OPENING_START") then
        CleanUIActionButtonsArtFrame:Hide();
    end
    if (event == "PET_BATTLE_CLOSE" and CleanUIData.ACTIONBARS_ACTIONBUTTON_BORDER) then
        CleanUIActionButtonsArtFrame:Show();
    end

    -- vehicles
    if (event == "UNIT_ENTERED_VEHICLE" and arg1 == "player" and UnitHasVehicleUI("player")) then
        CleanUIActionButtonsArtFrame:Hide();
    end
    if (event == "UNIT_EXITED_VEHICLE" and arg1 == "player" and CleanUIData.ACTIONBARS_ACTIONBUTTON_BORDER) then
        CleanUIActionButtonsArtFrame:Show();
    end
end

function CleanUI_ActionBarsHookEvents()
    hooksecurefunc("MultiActionBar_Update", CleanUI_MultiActionBar_Update);
    hooksecurefunc("StanceBar_UpdateState", CleanUI_StanceBar_UpdateState);
    hooksecurefunc("PetActionBar_Update", CleanUI_PetActionBar_Update);
    hooksecurefunc("ReputationWatchBar_Update", CleanUI_ReputationWatchBar_Update);
    hooksecurefunc("PetActionBar_UpdatePositionValues", CleanUI_PetActionBar_UpdatePositionValues);

    hooksecurefunc("ActionButton_OnUpdate", CleanUI_ActionButton_OnUpdate);
end

function CleanUI_ReorderActionButtons()
    MainMenuBar:SetWidth(772);

    MultiBarBottomRightButton7:ClearAllPoints();
    MultiBarBottomRightButton7:SetPoint("TOPLEFT", ActionButton12, "TOPRIGHT", 12, 0);
end

function CleanUI_ReorderExpBar()
    MainMenuExpBar:SetSize(765, 2);
    MainMenuExpBar:SetPoint("TOP", 0, -51);
    MainMenuExpBar.forceHideText = true;

    CleanUI_ReputationWatchBar_Update();

    ExhaustionLevelFillBar:ClearAllPoints();
    ExhaustionLevelFillBar:SetPoint("TOPLEFT", MainMenuExpBar, "TOPLEFT", 0, 0);
    ExhaustionLevelFillBar:SetPoint("BOTTOMRIGHT", MainMenuExpBar, "BOTTOMRIGHT", 0, 0);

    ExhaustionTick:Hide();
--ExhaustionTick:SetNormalTexture(nil);
--ExhaustionTick:SetHighlightTexture(nil);
end

function CleanUI_ReputationWatchBar_Update(newLevel)

    ReputationWatchBar:SetSize(765, 2);
    ReputationWatchStatusBar:SetSize(765, 2);
    ReputationWatchBar.forceHideText = true;
    ReputationWatchStatusBar.forceHideText = true;

    ReputationWatchBar:ClearAllPoints();
    ReputationWatchBar:SetPoint("TOP", CleanUIActionButtonsArtFrame, "TOP", 0, -4);

    ReputationWatchStatusBar:ClearAllPoints();
    ReputationWatchStatusBar:SetAllPoints(ReputationWatchBar);
    ReputationWatchStatusBarText:ClearAllPoints();
    ReputationWatchStatusBarText:SetAllPoints(ReputationWatchBar);
end
    
function CleanUI_ManageFramePositions()    
    UIPARENT_MANAGED_FRAME_POSITIONS["PETACTIONBAR_YPOS"].baseY = 87;
    UIPARENT_MANAGED_FRAME_POSITIONS["StanceBarFrame"].baseY = -11;
    UIPARENT_MANAGED_FRAME_POSITIONS["PossessBarFrame"].baseY = -11;
    UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarBottomLeft"].baseY = 6;
    
    for name, data in pairs(UIPARENT_MANAGED_FRAME_POSITIONS) do
        data.reputation = 0;
        data.maxLevel = 0;        
    end
    
    UIParent_ManageFramePositions();
end

function CleanUI_PetActionBar_UpdatePositionValues()
    if (PETACTIONBAR_XPOS >= 500) then
        PETACTIONBAR_XPOS = 323;
    end
end

function CleanUI_MainMenuBar_UpdateArt()

    if (not CleanUIData.ACTIONBARS_ACTIONBUTTON_BORDER) then
        CleanUIActionButtonsArtFrame:Hide();
        return;
    else
        CleanUIActionButtonsArtFrame:Show();
    end
end

function CleanUI_MultiActionBar_Update()
    local first = ActionButton1;

    if (MultiBarBottomLeft:IsShown()) then
        first = MultiBarBottomLeftButton1;
    end

    CleanUIActionButtonsArtFrame:ClearAllPoints();
    CleanUIActionButtonsArtFrame:SetPoint("TOPLEFT", first, "TOPLEFT", -8, 8);
    CleanUIActionButtonsArtFrame:SetPoint("BOTTOMRIGHT", ActionButton12, "BOTTOMRIGHT", 267, -10);
end

function CleanUI_StanceBar_UpdateState()

    if (not CleanUIData.ACTIONBARS_SHAPESHIFTBAR_BORDER) then
        CleanUIStanceBarArtFrame:Hide();
        return;
    else
        CleanUIStanceBarArtFrame:Show();
    end

    CleanUIStanceBarArtFrame:SetParent(_G["StanceButton1"]);
    CleanUIStanceBarArtFrame:SetFrameStrata("BACKGROUND");

    local first = _G["StanceButton1"];
    local last = _G["StanceButton1"];
    local button;

    for i = 1, 10 do
        button = _G["StanceButton"..i];
        if (button and button:IsVisible()) then
            last = button;
        end
    end

    CleanUIStanceBarArtFrame:ClearAllPoints();
    CleanUIStanceBarArtFrame:SetPoint("TOPLEFT", first, "TOPLEFT", -8, 6);
    CleanUIStanceBarArtFrame:SetPoint("BOTTOMRIGHT", last, "BOTTOMRIGHT", 8, -5);
end

function CleanUI_PetActionBar_Update()

    if (not CleanUIData.ACTIONBARS_PETACTIONBAR_BORDER) then
        CleanUIPetActionButtonBarArtFrame:Hide();
        return;
    else
        CleanUIPetActionButtonBarArtFrame:Show();
    end

    CleanUIPetActionButtonBarArtFrame:SetParent(_G["PetActionButton1"]);
    CleanUIPetActionButtonBarArtFrame:SetFrameStrata("BACKGROUND");

    local first = _G["PetActionButton1"];
    local last = _G["PetActionButton1"];
    local button;

    for i = 1, 10 do
        button = _G["PetActionButton"..i];
        if (button and button:IsVisible()) then
            last = button;
        end
    end

    CleanUIPetActionButtonBarArtFrame:ClearAllPoints();
    CleanUIPetActionButtonBarArtFrame:SetPoint("TOPLEFT", first, "TOPLEFT", -8, 6);
    CleanUIPetActionButtonBarArtFrame:SetPoint("BOTTOMRIGHT", last, "BOTTOMRIGHT", 8, -5);
end

function CleanUI_CleanActionButtons()

    if (not CleanUIData.CHANGE_ACTIONBARS) then
        return;
    end

    for i = 1, 12 do
        _G["MultiBarBottomLeftButton"..i.."FloatingBG"]:SetTexture(nil);
        _G["MultiBarBottomRightButton"..i.."FloatingBG"]:SetTexture(nil);
        _G["MultiBarRightButton"..i.."FloatingBG"]:SetTexture(nil);
        _G["MultiBarLeftButton"..i.."FloatingBG"]:SetTexture(nil);
    end

    if (not CleanUIData.ACTIONBARS_SHOWMACRONAMES) then
        for i = 1, 12 do
            _G["ActionButton"..i.."Name"]:Hide();
            _G["MultiBarBottomLeftButton"..i.."Name"]:Hide();
            _G["MultiBarBottomRightButton"..i.."Name"]:Hide();
            _G["MultiBarRightButton"..i.."Name"]:Hide();
            _G["MultiBarLeftButton"..i.."Name"]:Hide();
        end

        for i = 1, 10 do
            _G["StanceButton"..i.."Name"]:Hide();
        end
    end

    if (MASQUE and buttongroup) then
        if (CleanUIData.MASQUE_ACTIONBUTTONS) then
            CleanUI_SkinActionBars();
        end
    end
end

function CleanUI_SkinActionBars()
    for i = 1, 12 do
        buttongroup:AddButton(_G["ActionButton"..i]);
        buttongroup:AddButton(_G["MultiBarBottomLeftButton"..i]);
        buttongroup:AddButton(_G["MultiBarBottomRightButton"..i]);
        buttongroup:AddButton(_G["MultiBarRightButton"..i]);
        buttongroup:AddButton(_G["MultiBarLeftButton"..i]);
    end

    for i = 1, 10 do
        buttongroup:AddButton(_G["StanceButton"..i]);
        buttongroup:AddButton(_G["PetActionButton"..i]);
    end
end

function CleanUI_RemoveArtwork()
    MainMenuBarLeftEndCap:SetTexture();
    MainMenuBarRightEndCap:SetTexture();
    MainMenuBarLeftEndCap:Hide();
    MainMenuBarRightEndCap:Hide();

    MainMenuMaxLevelBar0:SetTexture();
    MainMenuMaxLevelBar1:SetTexture();
    MainMenuMaxLevelBar2:SetTexture();
    MainMenuMaxLevelBar3:SetTexture();
    MainMenuMaxLevelBar0:Hide();
    MainMenuMaxLevelBar1:Hide();
    MainMenuMaxLevelBar2:Hide();
    MainMenuMaxLevelBar3:Hide();

    MainMenuBarTexture0:SetTexture();
    MainMenuBarTexture1:SetTexture();
    MainMenuBarTexture2:SetTexture();
    MainMenuBarTexture3:SetTexture();
    MainMenuBarTexture0:Hide();
    MainMenuBarTexture1:Hide();
    MainMenuBarTexture2:Hide();
    MainMenuBarTexture3:Hide();

    MainMenuXPBarTextureLeftCap:SetTexture();
    MainMenuXPBarTextureRightCap:SetTexture();
    MainMenuXPBarTextureMid:SetTexture();
    MainMenuXPBarTextureLeftCap:Hide();
    MainMenuXPBarTextureRightCap:Hide();
    MainMenuXPBarTextureMid:Hide();

    for i=1,19 do
        local texture = _G["MainMenuXPBarDiv"..i];
        texture:SetTexture();
        texture:Hide();
    end

    PossessBackground1:SetTexture();
    PossessBackground2:SetTexture();
    PossessBackground1:Hide();
    PossessBackground2:Hide();

    SlidingActionBarTexture0:SetTexture();
    SlidingActionBarTexture1:SetTexture();
    SlidingActionBarTexture0:Hide();
    SlidingActionBarTexture1:Hide();

    StanceBarLeft:SetTexture();
    StanceBarRight:SetTexture();
    StanceBarMiddle:SetTexture();
    StanceBarLeft:Hide();
    StanceBarRight:Hide();
    StanceBarMiddle:Hide();

    ReputationWatchBarTexture0:SetTexture();
    ReputationWatchBarTexture1:SetTexture();
    ReputationWatchBarTexture2:SetTexture();
    ReputationWatchBarTexture3:SetTexture();
    ReputationWatchBarTexture0:Hide();
    ReputationWatchBarTexture1:Hide();
    ReputationWatchBarTexture2:Hide();
    ReputationWatchBarTexture3:Hide();

    ReputationXPBarTexture0:SetTexture();
    ReputationXPBarTexture1:SetTexture();
    ReputationXPBarTexture2:SetTexture();
    ReputationXPBarTexture3:SetTexture();
    ReputationXPBarTexture0:Hide();
    ReputationXPBarTexture1:Hide();
    ReputationXPBarTexture2:Hide();
    ReputationXPBarTexture3:Hide();

    MainMenuBarPageNumber:Hide();
end

function CleanUI_ChangeActionBarUpDownButtons()
    ActionBarDownButton:Hide();
    ActionBarUpButton:Hide();
end

function CleanUI_ChangeMicroButtons()
    for i=1, #MICRO_BUTTONS do
    --_G[MICRO_BUTTONS[i]]:GetNormalTexture():SetTexCoord(0.1, 0.9, 0.1, 0.9);
    end
end

function CleanUI_ActionBar_OnUpdate()
    if (not MainMenuBar.combopoints) then
        return;
    end

    -- combo points
    if (CleanUIData.ACTIONBARS_SHOWCOMBOPOINTS and UnitExists("target")) then
        MainMenuBar.combopoints:Show();
        CleanUI_ActionButtons_UpdateComboPoints(MainMenuBar.combopoints);
    else
        MainMenuBar.combopoints:Hide();
    end
end

function CleanUI_ActionButton_OnUpdate(self, elapsed)

    local rangeIndicator = _G[self:GetName().."CUIRangeIndicator"];
    if (not rangeIndicator) then
        rangeIndicator = self:CreateTexture(self:GetName().."CUIRangeIndicator", OVERLAY);
        rangeIndicator:SetAllPoints(self);
        rangeIndicator:SetTexture(1.0, 0.0, 0.0);
        rangeIndicator:SetBlendMode("MOD");
    end

    if (not CleanUIData.ACTIONBARS_SHOWRANGEINDICATOR) then
        rangeIndicator:Hide();
        return;
    end

    -- Handle range indicator
    local valid = IsActionInRange(self.action);
    if ( valid == false ) then
        rangeIndicator:Show();
    else
        rangeIndicator:Hide();
    end
end

function CleanUI_ActionButtons_UpdateComboPoints(parent)
    local combopoints = GetComboPoints("player", "target");
    for i=1, 5 do
        local combo = _G[parent:GetName()..i];

        if (i > combopoints) then
            combo:Hide();
        else
            combo:Show();
        end
    end
end
