local headerText;

function CleanUI_InitAuctionFrame()
    CleanUI_RegisterAuctionFrameEvents();
end

function CleanUI_RegisterAuctionFrameEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:HookScript("OnEvent", CleanUI_AuctionFrameOnEvent);
end

function CleanUI_AuctionFrameHookEvents()
    hooksecurefunc("AuctionFrameTab_OnClick", CleanUI_AuctionFrameTab_OnClick);
    hooksecurefunc("AuctionFrameBrowse_Update", CleanUI_AuctionFrameBrowse_Update);
    hooksecurefunc("FilterButton_SetType", CleanUI_FilterButton_SetType);
end

function CleanUI_AuctionFrameTab_OnClick(self, button, down, index)
    CleanUI_RemoveTextures(AuctionFrame);
    CleanUI_SetBackdrop(AuctionFrame);

    local index = self:GetID();

    if (index == 1) then
        headerText:SetText(BROWSE_AUCTIONS);
    elseif (index == 2) then
        headerText:SetText(BID);
    elseif (index == 3) then
        headerText:SetFormattedText(AUCTION_TITLE, UnitName("player"));
    end
end

function CleanUI_FilterButton_SetType(button, type, text, isLast)
    local normalTexture = _G[button:GetName().."NormalTexture"];
    local line = _G[button:GetName().."Lines"];
    local tex = button:GetNormalTexture();
    
    normalTexture:Hide();
    tex:SetTexture(nil);
end

function CleanUI_AuctionFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_AuctionUI" ) then
            CleanUI_AuctionFrameHookEvents();
            CleanUI_CleanAuctionFrame();
        end
    end
end

function CleanUI_CleanAuctionFrame()
    CleanUI_RemoveTextures(AuctionFrame);
    CleanUI_SetBackdrop(AuctionFrame);
    AuctionFrame:SetHeight(442);

    AuctionPortraitTexture:Hide();

    CleanUI_ChangeCloseButton(AuctionFrame);

    CleanUI_ChangeButton(AuctionsStackSizeMaxButton);
    CleanUI_ChangeButton(AuctionsNumStacksMaxButton);
    CleanUI_ChangeEditBox(AuctionsStackSizeEntry);
    CleanUI_ChangeEditBox(AuctionsNumStacksEntry);

    ExactMatchCheckButton:ClearAllPoints();
    ExactMatchCheckButton:SetPoint("TOP", ShowOnPlayerCheckButton, "BOTTOM", 0, 2);

    _, headerText = CleanUI_CreateHeaderFrame(AuctionFrame, "CUI_AuctionFrameHeaderBorderframe", 250, nil, BROWSE_AUCTIONS);
    BrowseTitle:Hide();
    BidTitle:Hide();
    AuctionsTitle:Hide();

    AuctionFrameTab1:SetPoint("TOPLEFT", AuctionFrame, "BOTTOMLEFT", 15, 2);

    local tabButton;
    for i=1, 10 do
        tabButton = _G["AuctionFrameTab"..i];
        if (not tabButton) then
            break;
        end

        CleanUI_ChangeTab(tabButton);
    end

    local filterButton;
    for i=1, 20 do
        filterButton = _G["AuctionFilterButton"..i];

        if (filterButton) then
            CleanUI_RemoveTextures(filterButton);
            CleanUI_ChangeFilterButton(filterButton);
        end
    end

    CleanUI_RemoveTextures(BrowsePrevPageButton);
    CleanUI_ChangeBackButton(BrowsePrevPageButton);

    CleanUI_RemoveTextures(BrowseNextPageButton);
    CleanUI_ChangeNextButton(BrowseNextPageButton);
    CleanUI_RemoveTextures(BrowseScrollFrame);
    CleanUI_RemoveTextures(BrowseFilterScrollFrame);
    CleanUI_ChangeButton(BrowseBidButton);
    CleanUI_ChangeButton(BrowseBuyoutButton);
    CleanUI_ChangeButtonRed(BrowseCloseButton);
    CleanUI_ChangeButton(BrowseSearchButton);
    CleanUI_ChangeButton(BrowseResetButton);

    CleanUI_ChangeDropDown(BrowseDropDown);

    BrowseNameText:SetPoint("TOPLEFT", 30, -20);

    BrowseLevelText:ClearAllPoints();
    BrowseLevelText:SetPoint("TOPLEFT", 180, -20);

    CleanUI_ChangeEditBox(BrowseName);
    BrowseName:SetPoint("TOPLEFT", BrowseNameText, "BOTTOMLEFT", 3, -5);
    CleanUI_ChangeEditBox(BrowseMinLevel);
    CleanUI_ChangeEditBox(BrowseMaxLevel);
    BrowseMinLevel:SetPoint("TOPLEFT", BrowseLevelText, "BOTTOMLEFT", 3, -5);

    BrowseSearchButton:ClearAllPoints();
    BrowseSearchButton:SetPoint("TOPLEFT", 30, -58);
    BrowseResetButton:ClearAllPoints();
    BrowseResetButton:SetPoint("LEFT", BrowseSearchButton, "RIGHT", 2, 0);

    BrowseBidPrice:SetPoint("BOTTOM", 25, 15);
    BrowseBidText:SetPoint("RIGHT", BrowseBidPrice, "LEFT", -10, 0);
    CleanUI_ChangeEditBox(BrowseBidPriceGold);
    CleanUI_ChangeEditBox(BrowseBidPriceSilver);
    CleanUI_ChangeEditBox(BrowseBidPriceCopper);

    CleanUI_ChangeFilterButton(BrowseQualitySort, true);
    CleanUI_ChangeFilterButton(BrowseLevelSort, true);
    CleanUI_ChangeFilterButton(BrowseDurationSort, true);
    CleanUI_ChangeFilterButton(BrowseHighBidderSort, true);
    CleanUI_ChangeFilterButton(BrowseCurrentBidSort, true);

    CleanUI_RemoveTextures(BidScrollFrame);
    CleanUI_ChangeButton(BidBidButton);
    CleanUI_ChangeButton(BidBuyoutButton);
    CleanUI_ChangeButtonRed(BidCloseButton);

    BidBidPrice:SetPoint("BOTTOM", 25, 15);
    BidBidText:SetPoint("RIGHT", BidBidPrice, "LEFT", -10, 0);
    CleanUI_ChangeEditBox(BidBidPriceGold);
    CleanUI_ChangeEditBox(BidBidPriceSilver);
    CleanUI_ChangeEditBox(BidBidPriceCopper);

    CleanUI_ChangeFilterButton(BidQualitySort, true);
    CleanUI_ChangeFilterButton(BidLevelSort, true);
    CleanUI_ChangeFilterButton(BidDurationSort, true);
    CleanUI_ChangeFilterButton(BidBuyoutSort, true);
    CleanUI_ChangeFilterButton(BidStatusSort, true);
    CleanUI_ChangeFilterButton(BidBidSort, true);

    CleanUI_RemoveTextures(AuctionsScrollFrame);
    CleanUI_ChangeButton(AuctionsCreateAuctionButton);
    CleanUI_ChangeButton(AuctionsCancelAuctionButton);
    CleanUI_ChangeButtonRed(AuctionsCloseButton);

    CleanUI_ChangeFilterButton(AuctionsQualitySort, true);
    CleanUI_ChangeFilterButton(AuctionsDurationSort, true);
    CleanUI_ChangeFilterButton(AuctionsHighBidderSort, true);
    CleanUI_ChangeFilterButton(AuctionsBidSort, true);

    CleanUI_ChangeDropDown(PriceDropDown);
    CleanUI_ChangeDropDown(DurationDropDown);

    CleanUI_ChangeEditBox(StartPriceGold);
    CleanUI_ChangeEditBox(StartPriceSilver);
    CleanUI_ChangeEditBox(StartPriceCopper);

    CleanUI_ChangeEditBox(BuyoutPriceGold);
    CleanUI_ChangeEditBox(BuyoutPriceSilver);
    CleanUI_ChangeEditBox(BuyoutPriceCopper);

    DurationDropDown:ClearAllPoints();
    DurationDropDown:SetPoint("BOTTOMLEFT", 67, 80);

    CleanUI_RemoveTextures(SideDressUpFrame);
    CleanUI_SetBackdrop(SideDressUpFrame);
    CleanUI_RemoveTextures(SideDressUpModelCloseButton);
    CleanUI_ChangeCloseButton(SideDressUpFrame, "SideDressUpModelCloseButton");
    CleanUI_ChangeButton(SideDressUpModelResetButton);

    local scanPetsButton = CreateFrame("Button", "CUIAuctionFrameScanPets", AuctionFrameBrowse, "UIPanelButtonTemplate");
    scanPetsButton:SetWidth(125);
    scanPetsButton:SetHeight(22);
    scanPetsButton:SetPoint("TOPRIGHT", AuctionFrameBrowse, "TOPRIGHT", 60, -25);
    scanPetsButton:SetText(CUI_PETSCAN_SCAN_PETS);
    scanPetsButton:SetScript("OnClick", CleanUI_AuctionFrameScanPets);
    scanPetsButton:SetFrameLevel(scanPetsButton:GetFrameLevel() + 1);
    CleanUI_ChangeButton(scanPetsButton);

    local scanResultFrame = CreateFrame("Frame", "CUIAuctionFrameScanResultsFrame", AuctionFrame, "CleanUIAuctionFrameScanResultsFrameTemplate");
    CleanUI_SetBackdrop(scanResultFrame);
    scanResultFrame:SetSize(335, 442);
    scanResultFrame:ClearAllPoints();
    scanResultFrame:SetPoint("TOPLEFT", AuctionFrame, "TOPRIGHT", -2, 0);
    CleanUI_CreateHeaderFrame(scanResultFrame, "CUI_AuctionFrameScanResultsFrameHeaderBorderframe", 150, nil, CUI_PETSCAN_FRAME_HEADER);
    CleanUI_ChangeCloseButton(scanResultFrame);

    -- create gui
    HybridScrollFrame_OnLoad(CUIAuctionFrameScanResultsFrame.scrollFrame);
    CUIAuctionFrameScanResultsFrame.scrollFrame.update = CleanUI_AuctionFrameScanResults_OnUpdate;
    HybridScrollFrame_CreateButtons(CUIAuctionFrameScanResultsFrame.scrollFrame, "CleanUIAuctionFrameScanResultEntryTemplate");
end

function CleanUI_AuctionFrameBrowse_Update()
    local offset = FauxScrollFrame_GetOffset(BrowseScrollFrame);

    local button, index, name, texture, count, quality, canUse, level, levelColHeader, minBid, minIncrement, buyoutPrice, bidAmount, highBidder, bidderFullName, owner, ownerFullName, saleStatus, itemId, hasAllInfo;
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType;
    local stats;

    for i=1, NUM_BROWSE_TO_DISPLAY do
        index = offset + i + (NUM_AUCTION_ITEMS_PER_PAGE * AuctionFrameBrowse.page);
        button = _G["BrowseButton"..i];

        if (not button.additionalInfo) then
            button.additionalInfo = button:CreateFontString(button:GetName().."AdditionalInfo", "ARTWORK");
            button.additionalInfo:SetFontObject("CuiFont_VerySmall");
            button.additionalInfo:SetWidth(200);
            button.additionalInfo:SetHeight(10);
            button.additionalInfo:SetTextColor(1.0, 1.0, 1.0, 1.0);
            button.additionalInfo:SetJustifyH("LEFT");
            button.additionalInfo:SetText("");
            button.additionalInfo:SetPoint("BOTTOMLEFT", button, "BOTTOMLEFT", 43, 5);
        end

        if (button:IsVisible()) then
            name, texture, count, quality, canUse, level, levelColHeader, minBid, minIncrement, buyoutPrice, bidAmount, highBidder, bidderFullName, owner, ownerFullName, saleStatus, itemId, hasAllInfo =  GetAuctionItemInfo("list", offset + i);
            itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType = GetItemInfo(itemId);

            stats = CleanUI_GetGemStats(itemId);
            button.additionalInfo:SetText(stats or "");
        end
    end
end

local tip = CreateFrame("GameTooltip", "GemStatsTip");
local line1 = tip:CreateFontString();
local line2 = tip:CreateFontString();
local line3 = tip:CreateFontString();

tip:AddFontStrings(tip:CreateFontString(), tip:CreateFontString());
tip:AddFontStrings(line1, tip:CreateFontString());
tip:AddFontStrings(line2, tip:CreateFontString());
tip:AddFontStrings(line3, tip:CreateFontString());

local GEM_S = "%+[0-9]+.*";

function CleanUI_GetGemStats(id)
    tip:SetOwner(WorldFrame, "ANCHOR_NONE");
    tip:SetItemByID(id);

    if(tip:IsShown()) then
        return string.match(line1:GetText() or '', GEM_S) or string.match(line2:GetText() or '', GEM_S) or string.match(line3:GetText() or '', GEM_S);
    end
end

local scannedPage;
local foundPets;
local notOwnedPets;
local auctionData;

local selectedIndex = nil;

function CleanUI_AuctionFrameScanPets()
    scannedPage = 1;
    foundPets = {};
    notOwnedPets = {};
    auctionData = {};
    CleanUI_PetScanAuctions();
end

function CleanUI_PetScanAuctions()
    CUIAuctionFrameScanResultsFrame:Show();
    CleanUI_AuctionFrameScanResults_OnUpdate();

    CUIAuctionFrameScanResultsFrame.info:SetText(CUI_PETSCAN_SCANNING_PAGE..scannedPage.." ...");

    local quality = UIDropDownMenu_GetSelectedValue(BrowseDropDown);
    QueryAuctionItems("", nil, nil, 0, 11, 0, scannedPage, 1, quality, 0);
    CleanUI_StartDelay(1, CleanUI_ScanAuctionPage);
end

function CleanUI_ScanAuctionPage()
    local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");
    if (numBatchAuctions > 0) then
        CleanUI_CheckPetAuctions(numBatchAuctions);

        scannedPage = scannedPage + 1;
        CleanUI_PetScanAuctions(scannedPage);
    else
        CleanUI_ScanPetJournalForAuctions();
    end
end

function CleanUI_CheckPetAuctions(count)
    for i=1, count do
        local name, texture, count, quality, canUse, level, levelColHeader, minBid, minIncrement, buyoutPrice, bidAmount, highBidder, bidderFullName, owner, ownerFullName, saleStatus, itemId, hasAllInfo = GetAuctionItemInfo("list", i);
        local itemLink = GetAuctionItemLink("list", i);
        
        if (tContains(foundPets, name..quality)) then            
            -- billigeres übernehmen
            if (minBid < auctionData[name..quality].minBid) then
                auctionData[name..quality] = {itemLink=itemLink, texture=texture, name=name, quality=quality, buyoutPrice=buyoutPrice, minBid=minBid};
            end
        end
        
        -- nicht gefundene hinzufügen
        if (not tContains(foundPets, name..quality)) then
            tinsert(foundPets, name..quality);
            auctionData[name..quality] = {itemLink=itemLink, texture=texture, name=name, quality=quality, buyoutPrice=buyoutPrice, minBid=minBid};
        end
    end
end

function CleanUI_ScanPetJournalForAuctions()

    local numPets, numOwned = C_PetJournal.GetNumPets(false);

    for index=1, numPets do
        local petID, speciesID, isOwned, customName, level, favorite, isRevoked, name = C_PetJournal.GetPetInfoByIndex(index);
        if (name) then
            if (not isOwned) then
                for q=0, 5 do
                    if (tContains(foundPets, name..q)) then
                        tinsert(notOwnedPets, name..q);
                    end
                end
            end
        end
    end

    CUIAuctionFrameScanResultsFrame.info:SetText(#notOwnedPets..CUI_PETSCAN_PETS_FOUND);
    CleanUI_AuctionFrameScanResults_OnUpdate();

    -- reset memory
    scannedPage = 1;
    foundPets = {};
end

function CleanUI_ToggleAuctionScanResultFrame()
    if (CUIAuctionFrameScanResultsFrame:IsVisible()) then
        CUIAuctionFrameScanResultsFrame:Hide();
    else
        CUIAuctionFrameScanResultsFrame:Show();
        CleanUI_AuctionFrameScanResults_OnUpdate();
    end
end

function CleanUI_AuctionFrameScanResults_OnUpdate()
    if (not CUIAuctionFrameScanResultsFrame) then
        return;
    end

    -- update group frame
    CUIAuctionFrameScanResultsFrame.scrollFrame.highlightFrame:Hide();

    local buttons = CUIAuctionFrameScanResultsFrame.scrollFrame.buttons;

    local numEntries = #notOwnedPets;
    local numButtons = #buttons;

    local scrollOffset = HybridScrollFrame_GetOffset(CUIAuctionFrameScanResultsFrame.scrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;

    local entry;
    local name;
    local minBid, buyoutPrice;

    for i=1, numButtons do
        local entryIndex = i + scrollOffset;
        entry = buttons[i];

        if ( entryIndex <= numEntries ) then
            name = notOwnedPets[entryIndex];

            entry:Show();
            entry.index = entryIndex;

            -- button data
            entry:SetText(auctionData[name].itemLink);

            entry.name = auctionData[name].name;

            entry.icon:SetTexture(auctionData[name].texture);

            minBid = GetCoinTextureString(auctionData[name].minBid);
            buyoutPrice = GetCoinTextureString(auctionData[name].buyoutPrice);

            entry.minBid:SetText(HIGHLIGHT_FONT_COLOR_CODE..minBid..FONT_COLOR_CODE_CLOSE);
            entry.buyoutPrice:SetText(BUYOUT_COST..": "..buyoutPrice);

            -- this isn't a header, hide the header textures
            entry:SetNormalTexture("");
            entry:SetHighlightTexture("");

            if (entryIndex == selectedIndex) then
                -- reposition highlight frames
                CUIAuctionFrameScanResultsFrame.scrollFrame.highlightFrame:SetParent(entry);
                CUIAuctionFrameScanResultsFrame.scrollFrame.highlightFrame:SetAllPoints(entry);
                CUIAuctionFrameScanResultsFrame.scrollFrame.highlightFrame:Show();
            end
        else
            entry:Hide();
        end

        displayedHeight = displayedHeight + buttonHeight;
    end

    HybridScrollFrame_Update(CUIAuctionFrameScanResultsFrame.scrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CleanUI_AuctionFrameScanResultEntry_OnClick(self, button, down)
    PlaySound("ACTIONBARBUTTONDOWN");
    selectedIndex = self.index;
    CleanUI_AuctionFrameScanResults_OnUpdate();
end

function CleanUI_AuctionFrameScanResultEntry_OnDoubleClick(self)
    QueryAuctionItems(self.name, nil, nil, 0, 11, 0, 0, 1, 0, 0);
end

