
-- temporary data
local item_profit = 0;

local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

local stackTable = {};

function CleanUI_InitBags()
    if (not CleanUIData.CHANGE_BAGS) then
        return;
    end

    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI", "Bags");
    end

    -- Display the amount of copper that is added to the tooltip
    GameTooltip:HookScript("OnTooltipAddMoney",
        function(self, amount)
            item_profit = amount;
        end);

    CleanUI_CleanBags();
    CleanUI_CleanBankFrame();
    CleanUI_ReorganizeButtons();

    CleanUI_BagsHookEvents();
    CleanUI_RegisterBagsEvents();

    CleanUI_CleanItemTextFrame();
end

function CleanUI_RegisterBagsEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");
    frame:RegisterEvent("MERCHANT_SHOW");
    frame:RegisterEvent("BANKFRAME_OPENED");
    frame:RegisterEvent("BAG_UPDATE");

    frame:HookScript("OnEvent", CleanUI_BagsOnEvent);
end

function CleanUI_BagsOnEvent(self, event, ...)

    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_GuildBankUI" ) then
            CleanUI_CleanGuildBankFrame();
            hooksecurefunc("GuildBankFrame_Update", function(button) CleanUI_GuildBankFrame_Update() end);
        end
    end

    if (event == "MERCHANT_SHOW") then
        CleanUI_MerchantShow();
    end

    if (event == "BANKFRAME_OPENED") then
        CleanUI_OpenAllBankBags();
    end

    if (event == "BAG_UPDATE") then
        CleanUI_BagUpdate();
    end
end

function CleanUI_BagsHookEvents()
    if (not CleanUIData.CHANGE_BAGS) then
        return;
    end

    hooksecurefunc("ContainerFrame_Update", function(frame) CleanUI_ContainerFrame_Update(frame) end);
    hooksecurefunc("BankFrameItemButton_Update", function(button) CleanUI_BankFrameItemButton_Update(button) end);
    hooksecurefunc("BankFrame_ShowPanel",  CleanUI_BankFrame_ShowPanel);

    hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", CleanUI_ContainerFrameItemButton_OnModifiedClick);

    hooksecurefunc("ContainerFrame_GenerateFrame", function(frame, size, id) CleanUI_ContainerFrame_GenerateFrame(frame, size, id) end);
end

function CleanUI_ContainerFrameItemButton_OnModifiedClick(self, button)
    -- CleanUIContainerLock
    if (IsAltKeyDown()) then
        local itemLink = GetContainerItemLink(self:GetParent():GetID(), self:GetID());

        if (itemLink) then
            -- lock state can't be removed by modified click
            if (not CleanUI_IsLockedItem(itemLink)) then
                CleanUI_AddLockedItem(itemLink);
            end
        end

        CleanUI_UpdateAllContainers();
    end
end

function CleanUI_IsLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            return true;
        end
    end

    return false;
end

function CleanUI_AddLockedItem(link)
    tinsert(CleanUIContainerLock, link);
end

function CleanUI_RemoveLockedItem(link)
    local itemName = GetItemInfo(link);
    local foundItemName;

    for index, foundLink in ipairs(CleanUIContainerLock) do
        foundItemName = GetItemInfo(foundLink);

        if (foundItemName == itemName) then
            tremove(CleanUIContainerLock, index);

            CleanUI_UpdateAllContainers();
            return true;
        end
    end
end

function CleanUI_UpdateAllContainers()
    for i=1, NUM_CONTAINER_FRAMES, 1 do
        local frame = _G["ContainerFrame"..i];
        if (frame and frame:IsShown()) then
            ContainerFrame_Update(frame);
        end
    end
end

function CleanUI_GetItemButtonName(itemButton)
    local name = itemButton:GetName();
    if (name) then
        return name;
    end

    name = itemButton:GetParent():GetName().."BagSlot"..itemButton:GetID();
    return name;
end

function CleanUI_CleanItemTextFrame()
    CleanUI_RemoveTextures(ItemTextFrame);
    CleanUI_SetBackdrop(ItemTextFrame);

    CleanUI_ChangeCloseButton(ItemTextFrame);

    CleanUI_CreateHeaderFrame(ItemTextFrame, "CUI_ItemTextFrameHeaderBorderframe", 175, ItemTextTitleText);

    CleanUI_RemoveTextures(ItemTextFrameInset);

    ItemTextScrollFrameTop:SetTexture(nil);
    ItemTextScrollFrameMiddle:SetTexture(nil);
    ItemTextScrollFrameBottom:SetTexture(nil);

    local background = ItemTextScrollFrame:CreateTexture("CUI_ItemTextFrameBackground", "BORDER");
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", ItemTextScrollFrame, "TOPLEFT", -15, 5);
    background:SetPoint("BOTTOMRIGHT", ItemTextScrollFrame, "BOTTOMRIGHT", 5, 0);

    background:SetTexture("interface\\Addons\\CleanUI\\skins\\frame\\paper-background");
end

function CleanUI_CleanBags()
    CleanUI_RemoveTextures(BackpackTokenFrame);

    CleanUI_ChangeEditBox(BagItemSearchBox);

    BackpackTokenFrameToken1Icon:SetSize(16, 16);
    BackpackTokenFrameToken1Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    BackpackTokenFrameToken2Icon:SetSize(16, 16);
    BackpackTokenFrameToken2Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
    BackpackTokenFrameToken3Icon:SetSize(16, 16);
    BackpackTokenFrameToken3Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);

    local stackButton = _G["CUI_STACK_BUTTON"];

    if (not stackButton) then
        stackButton = CreateFrame("Button", "CUI_STACK_BUTTON", ContainerFrame1, "UIPanelButtonTemplate");

        stackButton:SetText("+");
        CleanUI_RemoveUiButtonTexture(stackButton);

        stackButton:RegisterForClicks("AnyUp");

        stackButton:SetWidth(28);
        stackButton:SetHeight(28);
        stackButton:ClearAllPoints();
        stackButton:SetPoint("RIGHT", BagItemSearchBox, "LEFT", -4, 0);

        stackButton:HookScript("OnClick", CleanUI_StackButtonOnClick);
        stackButton:HookScript("OnEnter", CleanUI_ShowStackTooltip);
        stackButton:HookScript("OnLeave", CleanUI_HideStackTooltip);
    end

    CleanUI_RemoveTextures(ContainerFrame1PortraitButton);
    CleanUI_RemoveTextures(ContainerFrame2PortraitButton);
    CleanUI_RemoveTextures(ContainerFrame3PortraitButton);
    CleanUI_RemoveTextures(ContainerFrame4PortraitButton);
    CleanUI_RemoveTextures(ContainerFrame5PortraitButton);
end

function CleanUI_CleanBankFrame()
    CleanUI_RemoveTextures(BankFrame);
    CleanUI_RemoveTextures(BankSlotsFrame);

    BankPortraitTexture:Hide();

    CleanUI_ChangeEditBox(BankItemSearchBox);

    CleanUI_RemoveTextures(BankFrameMoneyFrameBorder);
    CleanUI_RemoveTextures(BankFrameMoneyFrameInset);

    CleanUI_SetBackdrop(BankFrame);
    CleanUI_ChangeCloseButton(BankFrame);

    CleanUI_CreateHeaderFrame(BankFrame, "CUI_BankFrameHeaderBorderframe", 200, BankFrameTitleText);

    CleanUI_ChangeButton(BankFramePurchaseButton);

    local tabButton;
    for i=1, 10 do
        tabButton = _G["BankFrameTab"..i];
        if (not tabButton) then
            break;
        end

        CleanUI_ChangeTab(tabButton);
    end
end

local reagentBankFrameClean = false;
function CleanUI_CleanReagentBankFrame()
    if (reagentBankFrameClean) then
        return;
    end

    CleanUI_RemoveTextures(ReagentBankFrame);
    CleanUI_ChangeButton(ReagentBankFrameUnlockInfoPurchaseButton);
    CleanUI_ChangeButton(ReagentBankFrame.DespositButton);

    reagentBankFrameClean = true;
end

function CleanUI_CleanGuildBankFrame()
    CleanUI_RemoveTextures(GuildBankFrame);
    CleanUI_SetBackdrop(GuildBankFrame);

    CleanUI_CreateHeaderFrame(GuildBankFrame, "CUI_GuildBankFrameHeaderBorderframe", 250, GuildBankTabTitle);

    local tabButton;
    for i=1, 10 do
        tabButton = _G["GuildBankFrameTab"..i];
        if (not tabButton) then
            break;
        end

        CleanUI_ChangeTab(tabButton);
    end

    local sideTabButton;
    for i=1, 10 do
        sideTabButton = _G["GuildBankTab"..i];
        if (not sideTabButton) then
            break;
        end

        CleanUI_ChangeSideTab(sideTabButton);
    end

    local column;
    for i=1, 10 do
        column = _G["GuildBankColumn"..i];
        if (not column) then
            break;
        end

        CleanUI_RemoveTextures(column);
    end

    local closeButton = CleanUI_GetChildOfTypeAtPos(GuildBankFrame, "Button", 0);

    if (closeButton) then
        CleanUI_ChangeCloseButton(GuildBankFrame, nil, closeButton);
    end

    GuildBankEmblemFrame:Hide();
    GuildBankMoneyFrameBackground:Hide();

    CleanUI_ChangeEditBox(GuildItemSearchBox);

    CleanUI_ChangeButton(GuildBankInfoSaveButton);
    CleanUI_ChangeButton(GuildBankFrameWithdrawButton);
    CleanUI_ChangeButton(GuildBankFrameDepositButton);

    CleanUI_RemoveTextures(GuildBankTransactionsScrollFrame);
    CleanUI_RemoveTextures(GuildBankInfoScrollFrame);
end

function CleanUI_ContainerFrame_GenerateFrame(frame, size, id)
    frame.size = size;
    local name = frame:GetName();

    _G[name.."BackgroundTop"]:SetTexture(nil);
    _G[name.."BackgroundMiddle1"]:SetTexture(nil);
    _G[name.."BackgroundMiddle2"]:SetTexture(nil);
    _G[name.."BackgroundBottom"]:SetTexture(nil);
    _G[name.."Portrait"]:Hide();
    --_G[name.."PortraitButton"]:Hide();

    local borderframe = _G[name.."Borderframe"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", name.."Borderframe", frame);

        CleanUI_SetBackdrop(borderframe);

        borderframe:ClearAllPoints();
        borderframe:SetPoint("TOPLEFT", frame, "TOPLEFT", 6, 0);
        borderframe:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0);

        borderframe:SetFrameStrata("LOW");
    end

    _G[name.."Name"]:SetParent(_G[name.."Borderframe"]);

    CleanUI_ChangeCloseButton(frame);

    local nameText = _G[name.."Name"];
    nameText:ClearAllPoints();
    nameText:SetPoint("TOPLEFT", frame, "TOPLEFT", 14, -10);
    nameText:SetWidth(150);
    nameText:SetJustifyH("LEFT");

    local portraitButton = _G[name.."PortraitButton"];
    portraitButton:ClearAllPoints();
    portraitButton:SetPoint("TOPLEFT", nameText, "TOPLEFT", 0, 0);
    portraitButton:SetPoint("BOTTOMRIGHT", nameText, "BOTTOMRIGHT", 0, 0);

    frame:SetFrameStrata("HIGH");
    borderframe:SetFrameStrata("HIGH");

    CleanUI_ChangeBackground(frame, size, id);
end

function CleanUI_ChangeBackground(frame, size, id)
    numberOfFreeSlots, bagType = GetContainerNumFreeSlots(id);

    local item;

    for i=1, size, 1 do
        item = _G[frame:GetName().."Item"..i];

        local background = _G[item:GetName().."Background"];

        if (not background) then
            background = item:CreateTexture(item:GetName().."Background", "BACKGROUND");
            background:SetSize(36, 36);
            background:ClearAllPoints();
            background:SetPoint("TOPLEFT", item, "TOPLEFT", 0, 0);
            background:SetPoint("BOTTOMRIGHT", item, "BOTTOMRIGHT", 0, 0);
            background:SetDrawLayer("BORDER");
        end

        if (bagType == 1) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\quiver");
        elseif (bagType == 2) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\ammo_pouch");
        elseif (bagType == 4) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\soul_bag");
        elseif (bagType == 8) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\leatherworking_bag");
        elseif (bagType == 16) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\inscription_bag");
        elseif (bagType == 32) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\herb_bag");
        elseif (bagType == 64) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\enchanting_bag");
        elseif (bagType == 128) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\engineering_bag");
        elseif (bagType == 256) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\keyring");
        elseif (bagType == 512) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\gem_bag");
        elseif (bagType == 1024) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\mining_bag");
        elseif (bagType == 2048) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\unknown");
        elseif (bagType == 4096) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\vanity_pets");
        elseif (bagType == 65536) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\cooking_bag");
        elseif (bagType == 1048576) then
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\tackle_box");
        else
            background:SetTexture("Interface\\AddOns\\CleanUI\\skins\\bag\\unspecified");
        end

        local itemLink = GetContainerItemLink(frame:GetID(), item:GetID());
        CleanUI_ToggleItemBackground(itemLink, item);
    end
end

function CleanUI_ReorganizeButtons()
    MainMenuBarBackpackButton:SetWidth(36);
    MainMenuBarBackpackButton:SetHeight(36);

    for i=0, 3, 1 do
        local button = getglobal("CharacterBag"..i.."Slot");
        button:SetWidth(24);
        button:SetHeight(24);
    end

    MainMenuBarBackpackButton:ClearAllPoints();
    MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", CleanUIActionButtonsArtFrame, "BOTTOMRIGHT", -7, 7);

    CharacterBag0Slot:ClearAllPoints();
    CharacterBag0Slot:SetPoint("RIGHT", MainMenuBarBackpackButton, "LEFT", -4, 0);
    CharacterBag1Slot:ClearAllPoints();
    CharacterBag1Slot:SetPoint("RIGHT", CharacterBag0Slot, "LEFT", -2, 0);
    CharacterBag2Slot:ClearAllPoints();
    CharacterBag2Slot:SetPoint("RIGHT", CharacterBag1Slot, "LEFT", -2, 0);
    CharacterBag3Slot:ClearAllPoints();
    CharacterBag3Slot:SetPoint("RIGHT", CharacterBag2Slot, "LEFT", -2, 0);

    if (MASQUE and buttongroup) then
        for i=0, 3, 1 do
            local button = getglobal("CharacterBag"..i.."Slot");
            local iconTexture = getglobal("CharacterBag"..i.."SlotIconTexture");
            iconTexture:SetDrawLayer("BACKGROUND");
            iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
            buttongroup:AddButton(button);
        end

        buttongroup:AddButton(MainMenuBarBackpackButton);
        local iconTexture = getglobal("MainMenuBarBackpackButtonIconTexture");
        iconTexture:SetDrawLayer("BACKGROUND");
        iconTexture:SetTexCoord(0.08, 0.92, 0.08, 0.92);
    end

    CleanUI_MoveBagButtons();
end

function CleanUI_MoveBagButtons()
    MainMenuBarBackpackButton:Hide();

    local parent = ContainerFrame1;

    CharacterBag0Slot:ClearAllPoints();
    CharacterBag0Slot:SetPoint("TOPRIGHT", parent, "BOTTOMRIGHT", -2, 0);

    for i=0, 3, 1 do
        local button = getglobal("CharacterBag"..i.."Slot");
        button:SetParent(parent);
    end
end

function CleanUI_BankFrame_ShowPanel(sidePanelName, selection)
    local frame = _G[sidePanelName];

    if (sidePanelName == "BankSlotsFrame") then
        CleanUI_ContainerFrame_Update(BankFrame);
    else
        CleanUI_ContainerFrame_Update(frame);
    end

    if (sidePanelName ~= "ReagentBankFrame") then
        return;
    end

    CleanUI_CleanReagentBankFrame();
end


function CleanUI_ContainerFrame_Update(frame)
    local id = frame:GetID();
    local name = frame:GetName();

    local itemButton;
    local texture, itemCount, locked, quality, readable;
    local tooltipOwner = GameTooltip:GetOwner();

    for i=1, frame.size, 1 do
        local itemButton = _G[name.."Item"..i];
        local itemLink = GetContainerItemLink(id, itemButton:GetID());

        CleanUI_UpdateItemLock(itemLink, itemButton);
        CleanUI_SetItemButtonBorderColor(itemLink, itemButton);
    end
end

function CleanUI_BankFrameItemButton_Update(itemButton)
    local container = itemButton:GetParent():GetID();
    local buttonID = itemButton:GetID();
    if( itemButton.isBag ) then
        container = -4;
    end

    local _, _, _, itemRarity, _, _, _, isFiltered = GetContainerItemInfo(container, buttonID);

    local itemButtonBorder = itemButton:GetNormalTexture();

    if (not itemButtonBorder) then
        return;
    end

    itemButton.IconBorder:Hide();

    itemButtonBorder:SetDrawLayer("ARTWORK");
    itemButtonBorder:Show();
    itemButtonBorder:SetAlpha(1);

    itemButtonBorder:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");

    if (itemRarity) then
        local r, g, b, hex = GetItemQualityColor(itemRarity);

        -- show white as gray
        if (itemRarity == 1 and CleanUIData.BAGS_SHOW_WHITE_AS_GRAY) then
            r, g, b = 0.5, 0.5, 0.5;
        end

        itemButtonBorder:SetVertexColor(r, g, b);
    else
        itemButtonBorder:SetVertexColor(0.5, 0.5, 0.5);
        itemButtonBorder:SetAlpha(0.3);
    end
end

function CleanUI_GuildBankFrame_Update()
    local button, index, column;
    for i=1, MAX_GUILDBANK_SLOTS_PER_TAB do
        index = mod(i, NUM_SLOTS_PER_GUILDBANK_GROUP);
        if ( index == 0 ) then
            index = NUM_SLOTS_PER_GUILDBANK_GROUP;
        end
        column = ceil((i-0.5)/NUM_SLOTS_PER_GUILDBANK_GROUP);
        button = _G["GuildBankColumn"..column.."Button"..index];

        local itemLink = GetGuildBankItemLink(GetCurrentGuildBankTab(), button:GetID());
        CleanUI_SetItemButtonBorderColor(itemLink, button);
    end
end

function CleanUI_ToggleItemBackground(itemLink, itemButton)
    local background = _G[CleanUI_GetItemButtonName(itemButton).."Background"];
    if (background) then
        if (itemLink) then
            background:Hide();
        else
            background:Show();
        end
    end
end

function CleanUI_UpdateItemLock(itemLink, itemButton)
    if (not itemLink) then
        CleanUI_UnLockItem(itemButton)
        return;
    end

    if (CleanUI_IsLockedItem(itemLink)) then
        CleanUI_LockItem(itemButton);
    else
        CleanUI_UnLockItem(itemButton);
    end
end

function CleanUI_LockItem(itemButton)
    --itemButton:Disable();

    itemButton:RegisterForClicks();
    itemButton:RegisterForDrag();

    if (not itemButton.unlockButton) then
        itemButton.unlockButton = CreateFrame("Button", nil, itemButton, "UIPanelButtonTemplate");
        CleanUI_RemoveUiButtonTexture(itemButton.unlockButton);
        itemButton.unlockButton:SetSize(12, 12);
        itemButton.unlockButton:SetNormalTexture("Interface\\Addons\\CleanUI\\skins\\bag\\lock");
        itemButton.unlockButton:GetNormalTexture():SetTexCoord(0, 1, 0, 1);
        itemButton.unlockButton:SetPoint("TOPLEFT", itemButton, "TOPLEFT", -1, 1);

        itemButton.unlockButton:SetScript("OnClick", CleanUI_UnLockClicked);
    end

    itemButton.unlockButton:Show();
end

function CleanUI_UnLockClicked(unlockButton)
    if (IsAltKeyDown()) then
        local itemLink = GetContainerItemLink(unlockButton:GetParent():GetParent():GetID(), unlockButton:GetParent():GetID());
        CleanUI_RemoveLockedItem(itemLink);
    end
end

function CleanUI_UnLockItem(itemButton)
    --itemButton:Enable();

    itemButton:RegisterForClicks("LeftButtonUp", "RightButtonUp");
    itemButton:RegisterForDrag("LeftButton");

    if (itemButton.unlockButton) then
        itemButton.unlockButton:Hide();
    end
end

function CleanUI_SetItemButtonBorderColor(itemLink, itemButton)
    itemButton.IconBorder:Hide();

    local itemLevelLabel = _G[CleanUI_GetItemButtonName(itemButton).."ItemLevel"];
    if (not itemLevelLabel) then
        itemLevelLabel = itemButton:CreateFontString(CleanUI_GetItemButtonName(itemButton).."ItemLevel", "ARTWORK");
        itemLevelLabel:SetFontObject("GameFontNormalSmall");
        itemLevelLabel:SetSize(itemButton:GetWidth(), 10);
        itemLevelLabel:ClearAllPoints();
        itemLevelLabel:SetPoint("TOPRIGHT", -2, -2);
        itemLevelLabel:SetJustifyH("RIGHT");
        itemLevelLabel:SetText("");
    end

    itemLevelLabel:Hide();

    CleanUI_ToggleItemBackground(itemLink, itemButton);

    local itemButtonBorder = itemButton:GetNormalTexture();

    if (not itemButtonBorder) then
        return;
    end

    itemButtonBorder:SetDrawLayer("ARTWORK");

    itemButtonBorder:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");

    if (not CleanUIData.BAGS_COLORIZE_ITEMS) then
        itemButtonBorder:SetVertexColor(0.5, 0.5, 0.5);
        return;
    end

    itemButtonBorder:Show();
    itemButtonBorder:SetAlpha(1);

    if (itemLink) then
        local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemLink);
        local adjustedItemLevel;

        if (itemType ~= "Quest" and itemRarity) then
            local r, g, b, hex = GetItemQualityColor(itemRarity);

            -- show white as gray
            if (itemRarity == 1 and CleanUIData.BAGS_SHOW_WHITE_AS_GRAY) then
                r, g, b = 0.5, 0.5, 0.5;
            end

            itemButtonBorder:SetVertexColor(r, g, b);
        else
            itemButtonBorder:SetVertexColor(0.5, 0.5, 0.5);
        end

        if (itemLevel) then
            -- filter
            if ( itemType == ARMOR or
                itemType == ENCHSLOT_WEAPON ) then
                adjustedItemLevel = CleanUI_GetActualItemLevel(itemLink);
                itemLevelLabel:SetText(ORANGE_FONT_COLOR_CODE..adjustedItemLevel..FONT_COLOR_CODE_CLOSE);
                itemLevelLabel:Show();
            end
        end
    else
        itemButtonBorder:SetVertexColor(0.5, 0.5, 0.5);
        itemButtonBorder:SetAlpha(0.3);
    end
end

function CleanUI_MerchantShow()
    -- auto repair
    if (not IsControlKeyDown() and CanMerchantRepair() and CleanUIData.BAGS_AUTOREPAIR) then
        repairAllCost, canRepair = GetRepairAllCost();

        if (IsAltKeyDown()) then
            RepairAllItems(1);      -- guild
        else
            RepairAllItems();
        end
        PlaySound("ITEM_REPAIR");

        if (repairAllCost > 0) then
            DEFAULT_CHAT_FRAME:AddMessage(CUI_BAGS_ALL_ITEMS_REPAIRED..". ("..GetCoinTextureString(repairAllCost)..")", 1.0, 1.0, 0, 1, 10);
        end
    end

    local sum_item_profit = 0;

    -- auto sell grey items
    if (CleanUIData.BAGS_AUTOSELLGRAY) then
        local container, containerid, numberOfSlots;
        local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture;
        for id = 0, 4, 1 do
            containerid = "ContainerFrame"..(id + 1);
            container = getglobal(containerid);

            if (container) then
                numberOfSlots = GetContainerNumSlots(id);

                for slot=1, numberOfSlots, 1 do
                    itemLink = GetContainerItemLink(id, numberOfSlots - slot + 1);

                    if (itemLink) then
                        itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);

                        if (itemRarity == 0) then
                            GameTooltip:SetBagItem(id, numberOfSlots - slot + 1);

                            if (item_profit and item_profit > 0) then
                                sum_item_profit = sum_item_profit + item_profit;
                                DEFAULT_CHAT_FRAME:AddMessage(itemLink.. " "..CUI_BAGS_ALL_ITEMS_SOLD..". ("..GetCoinTextureString(item_profit)..")", 1.0, 1.0, 0, 1, 10);
                                UseContainerItem(id, numberOfSlots - slot + 1);
                            end
                        end
                    end
                end
            end
        end

        if (sum_item_profit > 0) then
            DEFAULT_CHAT_FRAME:AddMessage(CUI_BAGS_TOTAL_AMOUNT..": "..GetCoinTextureString(sum_item_profit), 1.0, 1.0, 0, 1, 10);
        end
    end
end

function CleanUI_BagUpdate()
end

function CleanUI_OpenAllBankBags()
    if ( BankFrame:IsShown() ) then
        ToggleBag(5);
        ToggleBag(6);
        ToggleBag(7);
        ToggleBag(8);
        ToggleBag(9);
        ToggleBag(10);
        ToggleBag(11);
    end
end

function CleanUI_OpenAllBags()
    for i=0, 4 do
        OpenBag(i);
    end
end

function CleanUI_CloseAllBags()
    for i=0, 4 do
        CloseBag(i);
    end
end

function CleanUI_StackButtonOnClick()
    CleanUI_OpenAllBags();
    local bagData = CleanUI_CollectStackableItems();
    CleanUI_StackItems(bagData);
end

function CleanUI_CollectStackableItems()
    local bagData = {};

    local container, containerid, numberOfSlots;
    local texture, itemCount, locked, quality, readable, lootable, itemLink;
    local itemName, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture;
    local data;

    for id = 4, 0, -1 do
        containerid = "ContainerFrame"..(id + 1);
        container = getglobal(containerid);

        if (container) then
            numberOfSlots = GetContainerNumSlots(id);

            for slot=1, numberOfSlots, 1 do
                texture, itemCount, locked, quality, readable, lootable, itemLink = GetContainerItemInfo(id, slot);

                if (itemLink) then
                    itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemLink);

                    if (itemCount and itemStackCount and itemCount < itemStackCount) then
                        data = {id = id, slot = slot, itemName = itemName, itemCount = itemCount, itemStackCount = itemStackCount};
                        tinsert(bagData, data);
                    end
                end
            end
        end
    end

    return bagData;
end

function CleanUI_StackItems(bagData)
    local matchIndex, countMissing, countFound;
    for i=1, #bagData do
        matchIndex = CleanUI_FindStackableItemWithName(bagData, bagData[i].itemName, i + 1);

        if (matchIndex) then
            ClearCursor();
            countMissing = bagData[i].itemStackCount - bagData[i].itemCount;
            countFound = bagData[matchIndex].itemCount;

            if (countFound > countMissing) then
                -- get required amount
                SplitContainerItem(bagData[matchIndex].id, bagData[matchIndex].slot, countMissing);
                bagData[matchIndex].itemCount = bagData[matchIndex].itemCount - countMissing;
                bagData[i].itemCount = bagData[i].itemCount + countMissing;
            else
                -- get all
                PickupContainerItem(bagData[matchIndex].id, bagData[matchIndex].slot);
                bagData[matchIndex].itemCount = 0;
                bagData[i].itemCount = bagData[i].itemCount + countFound;
            end

            -- match it
            PickupContainerItem(bagData[i].id, bagData[i].slot);
        end
    end
end

function CleanUI_FindStackableItemWithName(bagData, name, toIndex)
    for i=#bagData, toIndex, -1 do
        if (bagData[i].itemName == name and bagData[i].itemCount > 0) then
            return i;
        end
    end

    return nil;
end

function CleanUI_ShowStackTooltip(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);
    GameTooltip:SetText("Stack Items", 1.0, 1.0, 1.0);
    GameTooltip:Show();
end

function CleanUI_HideStackTooltip()
    GameTooltip:FadeOut();
end
