
local old_scroll;

function CleanUI_InitGarrisonFrame()
    CleanUI_RegisterGarrisonFrameEvents();
end

function CleanUI_RegisterGarrisonFrameEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:HookScript("OnEvent", CleanUI_GarrisonFrameOnEvent);
end

function CleanUI_GarrisonFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if ( addonName == "Blizzard_GarrisonUI" ) then
            CleanUI_GarrisonFrameHookEvents();
            CleanUI_CleanGarrisonFrame();
            CleanUI_CleanGarrisonRecruiterFrame();
            CleanUI_CleanGarrisonCapacitiveDisplayFrame();
            
            --GARRISON_ANIMATION_LENGTH = 0;
        end
    end
end

function CleanUI_GarrisonFrameHookEvents()
    --hooksecurefunc("xx", CleanUI_xx);
end

function CleanUI_CleanGarrisonFrame()
    
    CleanUI_RemoveTextures(GarrisonMissionFrame);
    CleanUI_SetGraphicsBackground(GarrisonMissionFrame, nil, nil, 0, 0, 10, 0);
    
    GarrisonMissionFrame.CloseButton:ClearAllPoints();
    GarrisonMissionFrame.CloseButton:SetPoint("TOPRIGHT", GarrisonMissionFrame, "TOPRIGHT", -8, 0);

    local nextButton = CreateFrame("Button", "CUI_LoadNextMission", GarrisonMissionFrame.MissionTab.MissionPage.Stage, "UIPanelButtonTemplate");

    nextButton:RegisterForClicks("AnyUp");

    nextButton:SetWidth(24);
    nextButton:SetHeight(20);
    
    nextButton:SetText(">");
    
    nextButton:ClearAllPoints();
    nextButton:SetPoint("RIGHT", GarrisonMissionFrame.MissionTab.MissionPage.CloseButton, "LEFT", -20, 0);
    
    nextButton:HookScript("OnClick", CleanUI_OpenNextMission);

    local prevButton = CreateFrame("Button", "CUI_LoadPrevMission", GarrisonMissionFrame.MissionTab.MissionPage.Stage, "UIPanelButtonTemplate");

    prevButton:RegisterForClicks("AnyUp");

    prevButton:SetWidth(24);
    prevButton:SetHeight(20);

    prevButton:SetText("<");
    
    prevButton:ClearAllPoints();
    prevButton:SetPoint("RIGHT", nextButton, "LEFT", 0, 0);
    
    prevButton:HookScript("OnClick", CleanUI_OpenPreviousMission);
end

function CleanUI_OpenPreviousMission()
    local mission = CleanUI_GetPreviousMission();
    
    if (mission) then
        CleanUI_LoadMission(mission);
    end
end

function CleanUI_OpenNextMission()
    local mission = CleanUI_GetNextMission();
    
    if (mission) then
        CleanUI_LoadMission(mission);
    end
end

function CleanUI_LoadMission(mission)    
    PlaySound("UI_Garrison_CommandTable_SelectMission");
    
    GarrisonMissionPage_Close(nil);
    
    GarrisonMissionFrame.MissionTab.MissionList:Hide();
    GarrisonMissionFrame.MissionTab.MissionPage:Show();
    GarrisonMissionPage_ShowMission(mission);
    
    GarrisonMissionPage_ClearParty();
    
    GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID)
    GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
    GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
    
    GarrisonMissionPage_UpdateStartButton(GarrisonMissionFrame.MissionTab.MissionPage);
end

function CleanUI_GetPreviousMission()
    local missions = {};
    C_Garrison.GetAvailableMissions(missions);
    Garrison_SortMissions(missions);
    
    local self = GarrisonMissionFrame.MissionTab.MissionPage;
    local actMissionID = self.missionInfo.missionID;
    
    for i = 1, #missions do
        local mission = missions[i];
        
        if (mission.missionID == actMissionID and i > 1) then
            return missions[i-1];
        end
    end
    
    return nil;
end

function CleanUI_GetNextMission()
    local missions = {};
    C_Garrison.GetAvailableMissions(missions);
    Garrison_SortMissions(missions);
    
    local self = GarrisonMissionFrame.MissionTab.MissionPage;
    local actMissionID = self.missionInfo.missionID;
    
    for i = 1, #missions do
        local mission = missions[i];
        
        if (mission.missionID == actMissionID and #missions > i) then
            return missions[i+1];
        end
    end
    
    return nil;
end

function CleanUI_CleanGarrisonRecruiterFrame()
    CleanUI_RemoveTextures(GarrisonRecruiterFrame);
    CleanUI_RemoveTextures(GarrisonRecruiterFrameInset);
    CleanUI_SetBackdrop(GarrisonRecruiterFrame);
    
    CleanUI_ChangeCloseButton(GarrisonRecruiterFrame);
    GarrisonRecruiterFrame.PortraitTexture:Hide();
    CleanUI_ChangeButton(GarrisonRecruiterFrame.Pick.ChooseRecruits);
    CleanUI_ChangeButton(GarrisonRecruiterFrame.Random.ChooseRecruits);
    local button = CleanUI_GetChildOfTypeAtPos(GarrisonRecruiterFrame.UnavailableFrame, "Button", 0);
    CleanUI_ChangeButton(button);
    
    CleanUI_ChangeDropDown(GarrisonRecruiterFramePickThreatDropDown); 
end

function CleanUI_CleanGarrisonCapacitiveDisplayFrame()
    CleanUI_RemoveTextures(GarrisonCapacitiveDisplayFrame);
    CleanUI_RemoveTextures(GarrisonCapacitiveDisplayFrameInset);
    CleanUI_SetBackdrop(GarrisonCapacitiveDisplayFrame);
    
    CleanUI_CreateHeaderFrame(GarrisonCapacitiveDisplayFrame, "CUI_GarrisonCapacitiveDisplayFrameHeaderBorderframe", 250, GarrisonCapacitiveDisplayFrameTitleText);
    CleanUI_ChangeCloseButton(GarrisonCapacitiveDisplayFrame);
    
    -- move glow
    GarrisonCapacitiveDisplayFrame.FinishedGlow.FinishedFlare:SetTexture("Interface\\AddOns\\CleanUI\\skins\\frame\\finishedFlare");
    GarrisonCapacitiveDisplayFrame.FinishedGlow.FinishedFlare:ClearAllPoints();
    GarrisonCapacitiveDisplayFrame.FinishedGlow.FinishedFlare:SetPoint("TOPLEFT", GarrisonCapacitiveDisplayFrame, "TOPLEFT", 3, -3);
    GarrisonCapacitiveDisplayFrame.FinishedGlow.FinishedFlare:SetPoint("BOTTOMRIGHT", GarrisonCapacitiveDisplayFrame, "BOTTOMRIGHT", -3, 3);

    GarrisonCapacitiveDisplayFramePortrait:Hide();
    CleanUI_ChangeButton(GarrisonCapacitiveDisplayFrame.StartWorkOrderButton);
    CleanUI_ChangeButton(GarrisonCapacitiveDisplayFrame.CreateAllWorkOrdersButton);
    CleanUI_ChangeEditBox(GarrisonCapacitiveDisplayFrame.Count);
end