ZoneNames = {};

selectedWayPoint = nil;

function LoadZones(...)
    for i=1, arg.n, 1 do
        ZoneNames[i] = arg[i];
    end
end
local loaded = false;

function CleanUI_InitMap()

    local infoframe = _G["CleanUIMapFrame"];
    infoframe:ClearAllPoints();
    infoframe:SetAllPoints(WorldMapDetailFrame);

    local waypoints = _G["CleanUIMapWayPointFrame"];
    waypoints:ClearAllPoints();
    waypoints:SetAllPoints(WorldMapDetailFrame);

    local frame = CreateFrame("Frame", "CUI_MapUpdateFrame");
    frame:HookScript("OnUpdate", CleanUI_WorldMap_OnUpdate);

    CleanUI_MapHookEvents();
    CleanUI_RegisterMapEvents();
    CleanUI_CleanWorldMapUI();

    loaded = true;
end

function CleanUI_RegisterMapEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("WORLD_MAP_UPDATE");
    frame:RegisterEvent("PLAYER_LEVEL_UP");
    frame:RegisterEvent("CHAT_MSG_ADDON");

    frame:HookScript("OnEvent", CleanUI_MapOnEvent);
end

function CleanUI_MapOnEvent(self, event, ...)
    if (event == "WORLD_MAP_UPDATE") then
        local miniWorldMap = GetCVar("miniWorldMap");
        CleanUI_UpdateMap(miniWorldMap);
        CleanUI_ShowWayPoints();
    end
    if (event == "PLAYER_LEVEL_UP") then
        CleanUI_StartDelay(CUI_START_DELAY, CleanUI_InitMap);
    end
    if (event == "CHAT_MSG_ADDON") then
        local prefix, message, type, sender = ...;
        CleanUI_ReceiveWayPoint(prefix, message, type, sender);
    end
end

function CleanUI_MapHookEvents()
    hooksecurefunc("ProcessMapClick", CleanUI_ProcessMapClick);
    hooksecurefunc("WorldMap_ToggleSizeUp", CleanUI_WorldMap_ToggleSizeUp);
    hooksecurefunc("WorldMap_ToggleSizeDown", CleanUI_WorldMap_ToggleSizeDown);
    hooksecurefunc("WorldMapTrackingOptionsDropDown_Initialize", CleanUI_WorldMapTrackingOptionsDropDown_Initialize);
    
    hooksecurefunc("WorldMapFrame_AnimAlphaIn", CleanUI_WorldMapFrame_AnimAlphaIn);
    hooksecurefunc("WorldMapFrame_AnimAlphaOut", CleanUI_WorldMapFrame_AnimAlphaOut);

    UIDropDownMenu_Initialize(WorldMapFrame.UIElementsFrame.TrackingOptionsButton.DropDown, WorldMapTrackingOptionsDropDown_Initialize, "MENU");
end

function CleanUI_WorldMapFrame_AnimAlphaIn(self, useStartDelay)
    CUI_WorldMapFrameBorderFrameHeaderBorderframe:Show();
end

function CleanUI_WorldMapFrame_AnimAlphaOut(self, useStartDelay)
    CUI_WorldMapFrameBorderFrameHeaderBorderframe:Hide();
end

function CleanUI_CleanWorldMapUI()
    CleanUI_RemoveTextures(WorldMapFrame.BorderFrame);
    CleanUI_RemoveTextures(WorldMapFrame.BorderFrame.Inset);
    CleanUI_SetBackdrop(WorldMapFrame);

    CleanUI_ChangeCloseButton(WorldMapFrame.BorderFrame, nil, WorldMapFrameCloseButton);

    CleanUI_CreateHeaderFrame(WorldMapFrame.BorderFrame, "CUI_WorldMapFrameBorderFrameHeaderBorderframe", 250, WorldMapFrameTitleText);

    WorldMapFrameTutorialButton:Hide();

    WorldMapFrameNavBar:ClearAllPoints();
    WorldMapFrameNavBar:SetPoint("TOPLEFT", WorldMapFrame, "TOPLEFT", 15, -22);

    CleanUI_RemoveTextures(WorldMapFrameSizeUpButton);
    WorldMapFrameSizeUpButton:SetSize(22, 22);
    WorldMapFrameSizeUpButton:ClearAllPoints();
    WorldMapFrameSizeUpButton:SetPoint("RIGHT", WorldMapFrameCloseButton, "LEFT", 2, 0);
    WorldMapFrameSizeUpButton:SetNormalTexture("Interface\\Addons\\CleanUI\\skins\\ui\\sizeupbutton");
    WorldMapFrameSizeUpButton:SetPushedTexture("Interface\\Addons\\CleanUI\\skins\\ui\\sizeupbutton-pressed");
    WorldMapFrameSizeUpButton:SetHighlightTexture("Interface\\Addons\\CleanUI\\skins\\ui\\closebutton-hover");

    CleanUI_RemoveTextures(WorldMapFrameSizeDownButton);
    WorldMapFrameSizeDownButton:SetSize(22, 22);
    WorldMapFrameSizeDownButton:ClearAllPoints();
    WorldMapFrameSizeDownButton:SetPoint("RIGHT", WorldMapFrameCloseButton, "LEFT", -2, 0);
    WorldMapFrameSizeDownButton:SetNormalTexture("Interface\\Addons\\CleanUI\\skins\\ui\\sizedownbutton");
    WorldMapFrameSizeDownButton:SetPushedTexture("Interface\\Addons\\CleanUI\\skins\\ui\\sizedownbutton-pressed");
    WorldMapFrameSizeDownButton:SetHighlightTexture("Interface\\Addons\\CleanUI\\skins\\ui\\closebutton-hover");
    
    CleanUI_RemoveTextures(WorldMapFrameNavBar);
    CleanUI_RemoveTextures(WorldMapFrameNavBarOverlay);
end

function CleanUI_WorldMap_ToggleSizeUp()
    CleanUI_UpdateMap("0");
    CUI_WorldMapFrameBorderFrameHeaderBorderframe:Hide();
    WorldMapFrame:SetBackdrop(nil);

    --BlackoutWorld:Hide();
end

function CleanUI_WorldMap_ToggleSizeDown()
    CleanUI_UpdateMap("1");
    CUI_WorldMapFrameBorderFrameHeaderBorderframe:Show();
    CleanUI_SetBackdrop(WorldMapFrame);

    WorldMapFrameTutorialButton:Hide();

    WorldMapFrameNavBar:ClearAllPoints();
    WorldMapFrameNavBar:SetPoint("TOPLEFT", WorldMapFrame, "TOPLEFT", 15, -22);
end

function CleanUI_WorldMap_OnUpdate(self, elapsed)
    if (not WorldMapFrame:IsVisible()) then
        return;
    end

    local player_x, player_y = CleanUI_GetPlayerMapPosition();
    local cursor_x, cursor_y = CleanUI_GetCursorPosition();

    local output = "";

    if (cursor_x == nil or cursor_y == nil) then
        output = CUI_PLAYER..": "..player_x.."|cffffffff/|r"..player_y;
    else
        output = CUI_PLAYER..": "..player_x.."|cffffffff/|r"..player_y.." -- "..CUI_CURSOR..": "..cursor_x.."|cffffffff/|r"..cursor_y;
    end

    CleanUIWorldMapCoords:SetText(output);
end

function CleanUI_GetPlayerMapPosition()
    local x, y = GetPlayerMapPosition("player");

    x = x * 100;
    y = y * 100;

    return "|cffffffff"..string.format("%.1f", x).."|r", "|cffffffff"..string.format("%.1f", y).."|r";
end

function CleanUI_GetCursorPosition()

    local scale = WorldMapDetailFrame:GetEffectiveScale();
    local width = WorldMapDetailFrame:GetWidth();
    local height = WorldMapDetailFrame:GetHeight();
    local centerX, centerY = WorldMapDetailFrame:GetCenter();

    local x, y = GetCursorPosition();

    local adjustedX = (x / scale - (centerX - (width/2))) / width;
    local adjustedY = (centerY + (height/2) - y / scale) / height;

    adjustedX = adjustedX * 100;
    adjustedY = adjustedY * 100;

    if (adjustedX > 100 or adjustedY > 100) then
        return nil, nil, 0, 0;
    end

    return "|cffffffff"..string.format("%.1f", adjustedX).."|r", "|cffffffff"..string.format("%.1f", adjustedY).."|r", adjustedX, adjustedY;
end

function CleanUI_WorldMapTrackingOptionsDropDown_Initialize()
    local info = UIDropDownMenu_CreateInfo();

    -- way points
    local toggleWayPoints = CleanUIData.MAP_SHOWWAYPOINTS;

    info.text = CONFIG_TOGGLEWAYPOINTS;
    info.func = CleanUI_ToggleWayPoints;
    info.checked = toggleWayPoints;
    info.isNotRadio = true;
    info.keepShownOnClick = 1;
    UIDropDownMenu_AddButton(info);
end

function CleanUI_UpdateMap(miniWorldMap)

    if (not loaded) then
        return;
    end

    if (not WorldMapFrame:IsVisible()) then
        return;
    end

    if (miniWorldMap == "0") then
        -- full screen
    elseif (miniWorldMap == "1") then
        -- window
        CleanUIWorldMapCoords:ClearAllPoints();
        CleanUIWorldMapCoords:SetPoint("BOTTOMLEFT", WorldMapFrame, "BOTTOMLEFT", 10, 10);
    end

    -- Wegpunkte
    if (CleanUIData.MAP_SHOWWAYPOINTS) then
        CleanUI_ShowWayPoints();
    else
        CleanUI_HideWayPoints();
    end
end

function CleanUI_AddWaypoint(coords)
    local sx, sy, text = strsplit("/ ", coords, 3);

    if (sx and sy) then
        local x = tonumber(sx) / 100;
        local y = tonumber(sy) / 100;

        if (x >= 0 and x <= 1 and y >= 0 and y <= 1) then
            CleanUI_CreateWaypoint(x, y, text, "Interface\\TAXIFRAME\\UI-Taxi-Icon-Red");
        end
    end
end

function CleanUI_ProcessMapClick(adjustedX, adjustedY)
    CleanUI_ShowWayPoints();
    if (not IsControlKeyDown()) then
        return;
    end

    CleanUI_CreateWaypoint(adjustedX, adjustedY);
end

function CleanUI_CreateWaypointAtActPosition()
    SetMapToCurrentZone();
    local posX, posY = GetPlayerMapPosition("player");

    CleanUI_CreateWaypoint(posX, posY);
end

function CleanUI_CreateWaypoint(adjustedX, adjustedY, text, icon)
    local actContinent = GetCurrentMapContinent();
    local actArea = GetCurrentMapAreaID();
    local actLevel = GetCurrentMapDungeonLevel();

    local zones = { GetMapZones(actContinent) };
    local zoneIndex = GetCurrentMapZone();

    local actZone = zones[zoneIndex];

    text = text or date();
    icon = icon or "Interface\\GossipFrame\\GossipGossipIcon";

    local x = adjustedX * 100;
    local y = adjustedY * 100;

    local waypoint = {continent=actContinent, zone=actZone, area=actArea, level=actLevel, x=adjustedX * 100, y=adjustedY * 100, title=text, icon=icon};

    tinsert(CleanUIWayPoints, waypoint);

    CleanUI_ShowWayPoints();
end

function CleanUI_ShowWayPoints()

    local waypoints = _G["CleanUIMapWayPointFrame"];
    waypoints:Show();

    local frameWidth, frameHeight = waypoints:GetSize();

    local actContinent = GetCurrentMapContinent();
    local actArea = GetCurrentMapAreaID();
    local actLevel = GetCurrentMapDungeonLevel();

    local zones = { GetMapZones(actContinent) };
    local zoneIndex = GetCurrentMapZone();

    local actZone = zones[zoneIndex];

    local lastUsedIndex, continent, zone, area, level, x, y, title, icon;

    local index = 1;
    local lastUsedIndex = 1;
    for i, data in pairs(CleanUIWayPoints) do

        if (data) then
            continent = data.continent;
            zone = data.zone;
            area = data.area;
            level = data.level;
            x = data.x;
            y = data.y;
            title = data.title;
            if (data.icon) then
                icon = data.icon;
            else
                icon = "Interface\\GossipFrame\\GossipGossipIcon";
            end

            if (continent == actContinent and zone == actZone and area == actArea and level == actLevel) then
                local waypoint = _G["CleanUIMapWayPoint"..index];
                if (not waypoint) then
                    waypoint = CreateFrame("Button", "CleanUIMapWayPoint"..index, waypoints, "CleanUIMapWayPointTemplate");
                    waypoint:RegisterForClicks("AnyUp");
                    waypoint:SetMovable(true);
                end

                waypoint.icon:SetTexture(icon);

                waypoint.data = data;

                waypoint.info:SetText(title);
                waypoint.xy:SetText(string.format("%.1f", x).."/"..string.format("%.1f", y));

                local px = frameWidth * x / 100;
                local py = frameHeight * y / 100;

                waypoint:Show();
                waypoint:ClearAllPoints();
                waypoint:SetPoint("CENTER", waypoints, "TOPLEFT", px, -py);

                index = index + 1;
                lastUsedIndex = index;
            end
        end
    end

    -- hide unused waypoints
    while (lastUsedIndex) do
        local waypoint = _G["CleanUIMapWayPoint"..lastUsedIndex];
        if (waypoint) then
            waypoint:Hide();
            lastUsedIndex = lastUsedIndex + 1;
        else
            lastUsedIndex = nil;
            break;
        end
    end
end

function CleanUI_WayPoint_OnClick(self, button, down)
    local data = self.data;

    if (not data) then
        return;
    end

    if (button == "RightButton" and not IsControlKeyDown()) then
        selectedWayPoint = data;

        CleanUIMapWayPointContextMenu:ClearAllPoints();
        CleanUIMapWayPointContextMenu:SetPoint("TOPLEFT", self, "TOPLEFT", 10, -15);
        CleanUIMapWayPointContextMenu:Show();
    end
end

function CleanUI_WayPoint_OnMouseDown(frame, button)
    if (button == "RightButton") then
        if (IsControlKeyDown()) then
            frame:StartMoving();
        end
    end
end

function CleanUI_WayPoint_OnMouseUp(frame, button)
    if (button == "RightButton") then
        frame:StopMovingOrSizing();

        if (IsControlKeyDown()) then
            data = frame.data;
            local _, _, cursor_x, cursor_y = CleanUI_GetCursorPosition();
            data.x = cursor_x;
            data.y = cursor_y;
            CleanUI_ShowWayPoints();
        end
    end
end

function CleanUI_InitWayPointContextMenu(self)

    UIMenu_Initialize(self);
    UIMenu_AddButton(self, RENAME_GUILD, nil, function(frame) CleanUI_WayPointContextMenu_Rename() end);
    UIMenu_AddButton(self, EMBLEM_SYMBOL, nil, nil, "CleanUIMapWayPointContextMenuIcons");
    UIMenu_AddButton(self, SEND_LABEL, nil, nil, "CleanUIMapWayPointContextMenuSendTo");
    UIMenu_AddButton(self, DELETE, nil, function(frame) CleanUI_WayPointContextMenu_Delete() end);
    UIMenu_AddButton(self, "");
    UIMenu_AddButton(self, "reset", nil, function(frame) CleanUI_WayPointContextMenu_Reset() end);
    UIMenu_AddButton(self, "");
    UIMenu_AddButton(self, CANCEL, nil, function(frame) CleanUI_WayPointContextMenu_Cancel() end);
    UIMenu_AutoSize(self);
end

function CleanUI_InitWayPointContextMenuIcons(self)

    UIMenu_Initialize(self);
    UIMenu_AddButton(self, CHAT_DEFAULT, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\GossipFrame\\GossipGossipIcon") end);
    UIMenu_AddButton(self, "AvailableQuest", nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\GossipFrame\\AvailableQuestIcon") end);
    UIMenu_AddButton(self, "DailyAvailableQuest", nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\GossipFrame\\DailyQuestIcon") end);
    UIMenu_AddButton(self, TUTORIAL_TITLE35, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\GossipFrame\\TaxiGossipIcon") end);
    UIMenu_AddButton(self, TUTORIAL_TITLE14, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\GossipFrame\\TrainerGossipIcon") end);
    UIMenu_AddButton(self, TUTORIAL_TITLE20, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\GossipFrame\\VendorGossipIcon") end);
    UIMenu_AddButton(self, "Taxi gray", nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\TAXIFRAME\\UI-Taxi-Icon-Gray") end);
    UIMenu_AddButton(self, "Taxi green", nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\TAXIFRAME\\UI-Taxi-Icon-Green") end);
    UIMenu_AddButton(self, "Taxi red", nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\TAXIFRAME\\UI-Taxi-Icon-Red") end);
    UIMenu_AddButton(self, "Taxi white", nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\TAXIFRAME\\UI-Taxi-Icon-White") end);
    UIMenu_AddButton(self, "Taxi yellow", nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\TAXIFRAME\\UI-Taxi-Icon-Yellow") end);
    UIMenu_AddButton(self, INVTYPE_BAG, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\GROUPFRAME\\UI-Group-MasterLooter") end);
    UIMenu_AddButton(self, FACTION_ALLIANCE, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\WorldStateFrame\\AllianceIcon") end);
    UIMenu_AddButton(self, FACTION_HORDE, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\WorldStateFrame\\HordeIcon") end);
    UIMenu_AddButton(self, COMBAT, nil, function(frame) CleanUI_WayPointContextMenu_ChangeIcon("Interface\\TUTORIALFRAME\\UI-TutorialFrame-AttackCursor") end);
    UIMenu_AutoSize(self);
end

function CleanUI_InitWayPointContextMenuSendTo(self)

    UIMenu_Initialize(self);
    UIMenu_AddButton(self, CHAT_MSG_PARTY, nil, function(frame) CleanUI_WayPointContextMenu_Send(1) end);
    UIMenu_AddButton(self, CHAT_MSG_RAID, nil, function(frame) CleanUI_WayPointContextMenu_Send(2) end);
    UIMenu_AddButton(self, CHAT_MSG_GUILD, nil, function(frame) CleanUI_WayPointContextMenu_Send(3) end);
    UIMenu_AddButton(self, CHAT_MSG_BATTLEGROUND, nil, function(frame) CleanUI_WayPointContextMenu_Send(4) end);
    UIMenu_AddButton(self, CHAT_MSG_WHISPER_INFORM, nil, function(frame) CleanUI_WayPointContextMenu_Send(5) end);
    UIMenu_AutoSize(self);
end

function CleanUI_WayPointContextMenu_ChangeIcon(icon)
    local data = selectedWayPoint;

    if (not data) then
        return;
    end

    CleanUIMapWayPointContextMenu:Hide();
    CleanUIMapWayPointContextMenuIcons:Hide();

    data.icon = icon;
    CleanUI_ShowWayPoints();
end

function CleanUI_WayPointContextMenu_Rename()
    local data = selectedWayPoint;

    if (not data) then
        return;
    end

    CleanUIMapWayPointContextMenu:Hide();
    CleanUIMapWayPointContextMenuSendTo:Hide();

    CleanUIMapWayPointNameEditBox:SetText(data.title);
    CleanUIMapWayPointNameEditBox:Show();
end

function CleanUI_RenameWayPoint(name)
    local data = selectedWayPoint;

    if (not data) then
        return;
    end

    data.title = name;
    CleanUI_ShowWayPoints();
end

function CleanUI_WayPointContextMenu_Send(sendto)

    CleanUIMapWayPointContextMenu:Hide();
    CleanUIMapWayPointContextMenuSendTo:Hide();

    if (sendto == 1) then       -- group
        CleanUI_WayPointContextMenu_SendTo("PARTY");
    elseif (sendto == 2) then   -- raid
        CleanUI_WayPointContextMenu_SendTo("RAID");
    elseif (sendto == 3) then   -- guild
        CleanUI_WayPointContextMenu_SendTo("GUILD");
    elseif (sendto == 4) then   -- battleground
        CleanUI_WayPointContextMenu_SendTo("BATTLEGROUND");
    elseif (sendto == 5) then   -- whisper
    -- ...
    end
end

function CleanUI_WayPointContextMenu_SendTo(channel, name)
    local data = selectedWayPoint;

    if (not data) then
        return;
    end

    local message = data.continent..";"..data.zone..";"..data.area..";"..data.level..";"..data.x..";"..data.y..";"..data.title..";"..data.icon;
    CleanUI_SendWayPoint("CleanUI_WP", message, channel, name);
end

function CleanUI_SendWayPoint(prefix, message, channel, name)
    if (channel ~= "WHISPER") then
        SendAddonMessage(prefix, message, channel);
    else
        SendAddonMessage(prefix, message, channel, name);
    end
end

function CleanUI_ReceiveWayPoint(prefix, message, channel, sender)
    if (prefix == nil) then
        return;
    end

    if (prefix ~= "CleanUI_WP") then
        return;
    end

    local playername, _ = UnitName("player");

    local rcontinent, rzone, rarea, rlevel, strx, stry, rtitle, ricon = strsplit(";", message, 8);
    local waypoint = {continent=tonumber(rcontinent), zone=rzone, area=tonumber(rarea), level=tonumber(rlevel), x=tonumber(strx), y=tonumber(stry), title=rtitle, icon=ricon};

    if (sender == playername) then
        return;
    end

    CleanUI_AddReceivedWayPoint(waypoint);
    CleanUI_ShowWayPoints();
end

function CleanUI_AddReceivedWayPoint(waypoint)
    for index, found in pairs(CleanUIWayPoints) do
        if (waypoint.continent == found.continent and
            waypoint.zone == found.zone and
            waypoint.area == found.area and
            waypoint.level == found.level and
            waypoint.x == found.x and
            waypoint.y == found.y and
            waypoint.title == found.title) then
            -- keine doppelten Eintr�ge
            return;
        end
    end

    tinsert(CleanUIWayPoints, waypoint);
end

function CleanUI_WayPointContextMenu_Delete()

    CleanUIMapWayPointContextMenu:Hide();
    CleanUIMapWayPointContextMenuSendTo:Hide();

    for index, found in pairs(CleanUIWayPoints) do
        if (selectedWayPoint.zone == found.zone and
            selectedWayPoint.x == found.x and
            selectedWayPoint.y == found.y and
            selectedWayPoint.title == found.title) then

            table.remove(CleanUIWayPoints, index);
            CleanUI_ShowWayPoints();
            return;
        end
    end
end

function CleanUI_WayPointContextMenu_Reset()
    wipe(CleanUIWayPoints);
    CleanUI_ShowWayPoints();

    CleanUIMapWayPointContextMenu:Hide();
    CleanUIMapWayPointContextMenuSendTo:Hide();
end

function CleanUI_WayPointContextMenu_Cancel()
    CleanUIMapWayPointContextMenu:Hide();
    CleanUIMapWayPointContextMenuSendTo:Hide();
end

function CleanUI_HideWayPoints()
    local waypoints = _G["CleanUIMapWayPointFrame"];
    waypoints:Hide();
end

function CleanUI_ToggleMapLevel()
    CleanUIConfig_ToggleMapLevel();

    local miniWorldMap = GetCVar("miniWorldMap");
    CleanUI_UpdateMap(miniWorldMap);
end

function CleanUI_ToggleMapRoutes()
    CleanUIConfig_ToggleMapRoutes();

    local miniWorldMap = GetCVar("miniWorldMap");
    CleanUI_UpdateMap(miniWorldMap);
end

function CleanUI_ToggleWayPoints()
    CleanUIConfig_ToggleWayPoints();

    local miniWorldMap = GetCVar("miniWorldMap");
    CleanUI_UpdateMap(miniWorldMap);
end
