function CleanUI_InitMerchantFrame()
    CleanUI_MerchantFrameHookEvents();
    CleanUI_RegisterMerchantFrameEvents();

    -- DungeonBrowser stylen
    CleanUI_CleanMerchantFrame();
    CleanUI_CleanPetStableFrame();
end

function CleanUI_RegisterMerchantFrameEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:HookScript("OnEvent", CleanUI_MerchantFrameOnEvent);
end

function CleanUI_MerchantFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
    --if ( addonName == "*" ) then
    --end
    end
end

function CleanUI_MerchantFrameHookEvents()
    hooksecurefunc("MerchantFrame_UpdateMerchantInfo", CleanUI_MerchantFrame_UpdateMerchantInfo);
end

local gameTooltip = CreateFrame("GameTooltip", "CUI_GameTooltip_MerchantFrame", UIParent, "GameTooltipTemplate");

local function CleanUI_MerchantFrame_KnownItem(frame)
    gameTooltip:SetOwner(frame, "ANCHOR_RIGHT");
    gameTooltip:SetMerchantItem(frame:GetID());
    
    for line = gameTooltip:NumLines(), 1, -1 do
        if (string.find(_G["CUI_GameTooltip_MerchantFrameTextLeft"..line]:GetText(), ITEM_SPELL_KNOWN)) then
            gameTooltip:Hide();
            return true;
        end
    end
    
    gameTooltip:Hide();
    return false;
end

function CleanUI_MerchantFrame_UpdateMerchantInfo()
    local link, texture, item, itemButton;
    local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
    local r, g, b, hex;
    
    local known = false;

    local numMerchantItems = GetMerchantNumItems();

    for i=1, MERCHANT_ITEMS_PER_PAGE, 1 do
        local index = (((MerchantFrame.page - 1) * MERCHANT_ITEMS_PER_PAGE) + i);

        if ( index <= numMerchantItems ) then
            link = GetMerchantItemLink(index);
            
            item = _G['MerchantItem' .. i]
            itemButton = _G['MerchantItem' .. i .. 'ItemButton']
            texture = _G["MerchantItem"..i.."ItemButtonNormalTexture"];
            
            known = false;
        
            if (link) then
                itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(link);
                known = CleanUI_MerchantFrame_KnownItem(itemButton);

                r, g, b, hex = GetItemQualityColor(itemRarity);

                if (itemRarity == 1 and CleanUIData.BAGS_SHOW_WHITE_AS_GRAY) then
                    r, g, b = 0.5, 0.5, 0.5;
                end

                texture:SetVertexColor(r, g, b);
            else
                texture:SetVertexColor(0.5, 0.5, 0.5);
            end
  
            if (not itemButton.knownIcon) then
                itemButton.knownIcon = itemButton:CreateTexture("OVERLAY");
                itemButton.knownIcon:SetPoint("TOPLEFT", itemButton, "TOPLEFT", 4, -4);
                itemButton.knownIcon:SetPoint("BOTTOMRIGHT", itemButton, "BOTTOMRIGHT", -4, 4);
                itemButton.knownIcon:SetTexture("interface\\RaidFrame\\ReadyCheck-Ready");
            end
            
            if (known) then
                itemButton.knownIcon:Show();
            else
                itemButton.knownIcon:Hide();
            end
        end
    end
end

function CleanUI_CleanPetStableFrame()
    CleanUI_RemoveTextures(PetStableFrame);
    CleanUI_RemoveTextures(PetStableFrame.Inset);
    CleanUI_SetBackdrop(PetStableFrame);
    
    CleanUI_RemoveTextures(PetStableLeftInset);
    CleanUI_RemoveTextures(PetStableBottomInset);

    CleanUI_ChangeCloseButton(PetStableFrame);
    CleanUI_CreateHeaderFrame(PetStableFrame, "CUI_PetStableFrameHeaderBorderframe", 250, PetStableFrameTitleText);
end

function CleanUI_CleanMerchantFrame()
    CleanUI_RemoveTextures(MerchantFrame);
    CleanUI_SetBackdrop(MerchantFrame);

    MerchantFramePortrait:Hide();

    local tabButton;
    for i=1, 10 do
        tabButton = _G["MerchantFrameTab"..i];
        if (not tabButton) then
            break;
        end

        CleanUI_ChangeTab(tabButton, 1);
    end

    CleanUI_ChangeCloseButton(MerchantFrame);

    CleanUI_CreateHeaderFrame(MerchantFrame, "CUI_MerchantFrameHeaderBorderframe", 250, MerchantNameText);

    CleanUI_RemoveTextures(MerchantFrameInset);

    CleanUI_RemoveTextures(MerchantFrameLootFilter);

    CleanUI_RemoveTextures(MerchantMoneyInset);
    MerchantMoneyBg:Hide();

    CleanUI_RemoveTextures(MerchantMoneyFrame);
    CleanUI_RemoveTextures(MerchantExtraCurrencyInset);
    CleanUI_RemoveTextures(MerchantExtraCurrencyBg);

    local item;
    for i=1, 12 do
        item = _G["MerchantItem"..i];
        if (not item) then
            break;
        end

        CleanUI_RemoveTextures(item);

        _G["MerchantItem"..i.."ItemButtonNormalTexture"]:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
    end

    CleanUI_RemoveTextures(MerchantBuyBackItem);
    _G["MerchantBuyBackItemItemButtonNormalTexture"]:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
    _G["MerchantBuyBackItemItemButtonNormalTexture"]:SetVertexColor(0.5, 0.5, 0.5);

    CleanUI_ChangePrevButton(MerchantPrevPageButton);
    CleanUI_ChangeNextButton(MerchantNextPageButton);

    CleanUI_RemoveTextures(MerchantPrevPageButton);
    CleanUI_ChangeBackButton(MerchantPrevPageButton);

    CleanUI_RemoveTextures(MerchantNextPageButton);
    CleanUI_ChangeNextButton(MerchantNextPageButton);
end
