local namePlateStatusBars = {};

function CleanUI_InitNameplates()
    local frame = CreateFrame("Frame");
    frame:HookScript("OnUpdate", CleanUI_NameplatesOnUpdate);

    frame:RegisterEvent("PLAYER_REGEN_DISABLED");
    frame:RegisterEvent("PLAYER_REGEN_ENABLED");

    frame:HookScript("OnEvent", CleanUI_NameplatesOnEvent);

    SetCVar("bloattest", 0);
    SetCVar("bloatnameplates", 1);
    SetCVar("bloatthreat", 1);

    CleanUI_DeactivateNameplates();
end

local UPDATE_WAIT = 0.1;
local _WaitTimeSinceLast = 0;
function CleanUI_NameplatesOnUpdate(self, elapsed)
    if (not CleanUIData.CHANGE_NAMEPLATES) then
        return;
    end

    if (_WaitTimeSinceLast <= 0) then
        CleanUI_UpdateNameplates();
        _WaitTimeSinceLast = UPDATE_WAIT;
    else
        _WaitTimeSinceLast = _WaitTimeSinceLast - elapsed;
    end
end

function CleanUI_NameplatesOnEvent(self, event, ...)
    if (event == "PLAYER_REGEN_DISABLED") then
        CleanUI_ActivateNameplates();
    end
    if (event == "PLAYER_REGEN_ENABLED") then
        CleanUI_DeactivateNameplates();
    end
end

function CleanUI_ActivateNameplates()
    if (CleanUIData.NAMEPLATES_COMBATONLY) then
        SetCVar("nameplateShowEnemies", 1);
    end
end

function CleanUI_DeactivateNameplates()
    if (CleanUIData.NAMEPLATES_COMBATONLY) then
        SetCVar("nameplateShowEnemies", 0);
    end
end

function CleanUI_CheckActivateNameplates()
    if (not InCombatLockdown()) then
        SetCVar("nameplateShowEnemies", 0);
    else
        SetCVar("nameplateShowEnemies", 1);
    end
end

function CleanUI_UpdateNameplates()
    for i=1, select("#", WorldFrame:GetChildren()) do
        local frame = select(i, WorldFrame:GetChildren());
        if(frame:GetName() and string.find(frame:GetName(), "NamePlate")) then
            if (not frame.initialized) then
                CleanUI_Nameplates_InitFrame(frame);
            end

            CleanUI_Nameplates_UpdateFrame(frame);
        end
    end
end

function CleanUI_Nameplates_OnShow(frame)
    local r, g, b = frame.barFrame.healthbar:GetStatusBarColor();
    frame.barFrame.healthbar.cuiBar:SetStatusBarColor(r, g, b);

    r, g, b = frame.barFrame.castbar:GetStatusBarColor();
    frame.barFrame.castbar.cuiBar:SetStatusBarColor(r, g, b);

    CleanUI_Nameplates_UpdateTextures(frame);
end

function CleanUI_Nameplates_OnHide(frame)
end

function CleanUI_Nameplates_HealthOnValueChanged(healthBar)
    local value = healthBar:GetValue();
    healthBar.cuiBar:SetValue(value);

    if (not CleanUIData.NAMEPLATES_PERCENT) then
        healthBar.cuiBar.text:SetText("");
    else
        local value = healthBar:GetValue();
        local maxValue = select(2, healthBar:GetMinMaxValues());

        local healthtype = "percent";
        if(healthtype == "minmax") then
            healthBar.cuiBar.text:SetFormattedText("%d / %d", value, maxValue);
        elseif(healthtype == "percent") then
            healthBar.cuiBar.text:SetFormattedText("%d%%", value / maxValue * 100);
        else
            healthBar.cuiBar.text:SetText("");
        end
    end
end

function CleanUI_Nameplates_HealthOnMinMaxChanged(healthBar)
    local min, max = healthBar:GetMinMaxValues();
    healthBar.cuiBar:SetMinMaxValues(min, max);
end

function CleanUI_Nameplates_CastOnMinMaxChanged(castBar)
    local min, max = castBar:GetMinMaxValues();
    castBar.cuiBar:SetMinMaxValues(min, max);
end

function CleanUI_Nameplates_CastOnValueChanged(castBar)
    local value = castBar:GetValue();
    castBar.cuiBar:SetValue(value);
end

function CleanUI_Nameplates_InitRegions(frame)
    frame.barFrame, frame.nameFrame = frame:GetChildren();
    
    frame.barFrame.threat, frame.barFrame.border, frame.barFrame.highlight, frame.barFrame.level, frame.barFrame.boss, frame.barFrame.raid, frame.barFrame.dragon = frame.barFrame:GetRegions();
    frame.nameFrame.name = frame.nameFrame:GetRegions();
    
    frame.barFrame.healthbar, frame.barFrame.castbar = frame.barFrame:GetChildren();
    frame.barFrame.healthbar.texture =  frame.barFrame.healthbar:GetRegions();
    frame.barFrame.castbar.texture, frame.barFrame.castbar.border, frame.barFrame.castbar.shield, frame.barFrame.castbar.icon, frame.barFrame.castbar.text, frame.barFrame.castbar.shadow =  frame.barFrame.castbar:GetRegions();
end

function CleanUI_Nameplates_InitFrame(frame)
    frame.initialized = true;

    CleanUI_Nameplates_InitRegions(frame);

    frame:HookScript("OnShow", CleanUI_Nameplates_OnShow);
    frame:HookScript("OnHide", CleanUI_Nameplates_OnHide);

    frame.barFrame.healthbar:HookScript("OnValueChanged", CleanUI_Nameplates_HealthOnValueChanged);
    frame.barFrame.healthbar:HookScript("OnMinMaxChanged", CleanUI_Nameplates_HealthOnMinMaxChanged);
    frame.barFrame.healthbar:SetStatusBarTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\empty");

    local cuiHealthBar = CreateFrame("StatusBar", frame:GetName().."CUIHealthBar", frame.barFrame.healthbar, "CleanUINamePlatesStatusBarTemplate");
    cuiHealthBar:SetSize(100, 6);
    cuiHealthBar:SetPoint("CENTER", frame, "CENTER", 0, 0);
    cuiHealthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    cuiHealthBar:SetBackdrop( { bgFile = CUI_FRAME_BACKGROUND } );
    cuiHealthBar:SetBackdropColor(0.0, 0.0, 0.0, 1.0);

    cuiHealthBar.text:SetDrawLayer("ARTWORK");

    frame.barFrame.healthbar.cuiBar = cuiHealthBar;

    namePlateStatusBars[frame:GetName().."HealthBar"] = frame.barFrame.healthbar.cuiBar;

    frame.nameFrame.name:SetFontObject("CuiFont_NamePlates_Name");
    frame.barFrame.level:SetFontObject("CuiFont_NamePlates_Level");

    CleanUI_Nameplates_HealthOnMinMaxChanged(frame.barFrame.healthbar);
    CleanUI_Nameplates_HealthOnValueChanged(frame.barFrame.healthbar);

    -- hasfocus
    frame.focus = cuiHealthBar:CreateTexture(nil, "BACKGROUND");
    frame.focus:SetPoint("TOPLEFT", cuiHealthBar, "TOPLEFT", -1, 1);
    frame.focus:SetPoint("BOTTOMRIGHT", cuiHealthBar, "BOTTOMRIGHT", 1, -1);
    frame.focus:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\focus");
    frame.focus:Hide();

    -- pvp
    frame.pvp = cuiHealthBar:CreateTexture(nil, "BACKGROUND");
    frame.pvp:SetPoint("RIGHT", cuiHealthBar, "LEFT", 0, 0);
    frame.pvp:SetSize(20, 20);
    frame.pvp:Hide();

    -- ComboPoints
    frame.combopoints = CreateFrame("Frame", frame:GetName().."CUIComboPoints", frame);
    frame.combopoints:SetSize(100, 10);
    frame.combopoints:ClearAllPoints();
    frame.combopoints:SetPoint("TOPLEFT", cuiHealthBar, "BOTTOMLEFT", 0, -2);

    CleanUI_InitNameplateCombopoints(frame.combopoints);
    frame.combopoints:Hide();

    -- castbar
    local cuiCastBar = CreateFrame("StatusBar", frame:GetName().."CUICastBar", frame.barFrame.castbar, "CleanUINamePlatesStatusBarTemplate");
    cuiCastBar:SetSize(75, 4);
    cuiCastBar:SetPoint("TOPRIGHT", cuiHealthBar, "BOTTOMRIGHT", 0, -2);
    cuiCastBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

    cuiCastBar:SetBackdrop( { bgFile = CUI_FRAME_BACKGROUND } );
    cuiCastBar:SetBackdropColor(0.0, 0.0, 0.0, 1.0);
    frame.barFrame.castbar.cuiBar = cuiCastBar;

    frame.barFrame.castbar:HookScript("OnValueChanged", CleanUI_Nameplates_CastOnValueChanged);
    frame.barFrame.castbar:HookScript("OnMinMaxChanged", CleanUI_Nameplates_CastOnMinMaxChanged);
    frame.barFrame.castbar:SetStatusBarTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\empty");

    frame.barFrame.castbar.text:ClearAllPoints();
    frame.barFrame.castbar.text:SetAllPoints(frame.barFrame.castbar.shadow);

    frame.barFrame.castbar.shadow:ClearAllPoints();
    frame.barFrame.castbar.shadow:SetPoint("TOPLEFT", cuiCastBar, "BOTTOMLEFT", 0, -2);
    frame.barFrame.castbar.shadow:SetPoint("BOTTOMRIGHT", cuiCastBar, "BOTTOMRIGHT", 0, -18);

    -- interrupt
    cuiCastBar.interrupt = cuiCastBar:CreateTexture(nil, "BACKGROUND");
    cuiCastBar.interrupt:SetPoint("TOPLEFT", cuiCastBar, "TOPLEFT", -1, 1);
    cuiCastBar.interrupt:SetPoint("BOTTOMRIGHT", cuiCastBar, "BOTTOMRIGHT", 1, -1);
    cuiCastBar.interrupt:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\focus");
    cuiCastBar.interrupt:SetVertexColor(0.8, 0.0, 0.0);
    cuiCastBar.interrupt:Hide();

    -- dragon
    frame.barFrame.healthbar.cuiBar.dragon = frame.barFrame.healthbar.cuiBar:CreateTexture(nil, "BACKGROUND");
    frame.barFrame.healthbar.cuiBar.dragon:SetPoint("CENTER", cuiCastBar, "RIGHT", -2.5, 7);
    frame.barFrame.healthbar.cuiBar.dragon:SetSize(24, 24);
    frame.barFrame.healthbar.cuiBar.dragon:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\elite");
    frame.barFrame.healthbar.cuiBar.dragon:Hide();

    namePlateStatusBars[frame:GetName().."CastBar"] = frame.barFrame.castbar.cuiBar;

    CleanUI_Nameplates_OnShow(frame);
end

local function getColor(r, g, b)
    return floor(r*100 + .5)/100, floor(g*100 + .5)/100, floor(b*100 + .5)/100
end

function CleanUI_Nameplates_UpdateTextures(frame)
    frame.nameFrame.name:ClearAllPoints();
    frame.nameFrame.name:SetPoint("BOTTOM", frame.barFrame.healthbar.cuiBar, "TOP", 0, 3);

    if (frame.focus) then
        frame.focus:ClearAllPoints();
        frame.focus:SetPoint("TOPLEFT", frame.barFrame.healthbar.cuiBar, "TOPLEFT", -1, 1);
        frame.focus:SetPoint("BOTTOMRIGHT", frame.barFrame.healthbar.cuiBar, "BOTTOMRIGHT", 1, -1);
        frame.focus:SetDrawLayer("BORDER");
    end

    if (frame.barFrame.level) then
        frame.barFrame.level:ClearAllPoints();
        frame.barFrame.level:SetPoint("TOPLEFT", frame.barFrame.healthbar.cuiBar, "TOPRIGHT", -20, 0);
        frame.barFrame.level:SetPoint("BOTTOMRIGHT", frame.barFrame.healthbar.cuiBar, "BOTTOMRIGHT", 2, 0);
        frame.barFrame.level:SetParent(frame.barFrame.healthbar.cuiBar);
    end

    if (frame.barFrame.boss) then
        frame.barFrame.boss:SetSize(12, 12);
        frame.barFrame.boss:ClearAllPoints();
        frame.barFrame.boss:SetPoint("RIGHT", frame.barFrame.healthbar.cuiBar, "RIGHT", 0, 0);
        frame.barFrame.boss:SetParent(frame.barFrame.healthbar.cuiBar);
    end

    if (frame.barFrame.border) then
        frame.barFrame.border:SetTexture(nil);
        frame.barFrame.border:SetTexCoord(0, 1, 0, 1);
        frame.barFrame.border:ClearAllPoints();
        frame.barFrame.border:SetAllPoints(frame.barFrame.healthbar.cuiBar);
        frame.barFrame.border:SetParent(frame.barFrame.healthbar.cuiBar);
    end

    if (frame.barFrame.highlight) then
        frame.barFrame.highlight:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\highlight");
        frame.barFrame.highlight:SetTexCoord(0, 1, 0, 1);
        frame.barFrame.highlight:SetDrawLayer("BORDER");
        frame.barFrame.highlight:SetParent(frame.barFrame.healthbar.cuiBar);
    end

    if (frame.barFrame.threat) then
        frame.barFrame.threat:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\threat");
        frame.barFrame.threat:SetTexCoord(0, 1, 0, 1);
    end

    if (frame.barFrame.raid) then
        frame.barFrame.raid:SetSize(12, 12);
        frame.barFrame.raid:ClearAllPoints();
        frame.barFrame.raid:SetPoint("RIGHT", frame.barFrame.healthbar.cuiBar, "LEFT", -2, 0);
        frame.barFrame.raid:SetParent(frame.barFrame.healthbar.cuiBar);
    end
    
    if (frame.barFrame.dragon) then
        if (frame.barFrame.dragon:IsVisible()) then
            frame.barFrame.healthbar.cuiBar.dragon:Show();
        else
            frame.barFrame.healthbar.cuiBar.dragon:Hide();
        end
        
        local r, g, b = getColor(frame.barFrame.dragon:GetVertexColor())
        CleanUI_RemoveTexture(frame.barFrame.dragon);
        
        -- elite
        if (frame.barFrame.boss:IsVisible()) then
            frame.barFrame.healthbar.cuiBar.dragon:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\boss");
        else
            frame.barFrame.healthbar.cuiBar.dragon:SetTexture("Interface\\AddOns\\CleanUI\\skins\\nameplates\\elite");
        end
    end
    
    CleanUI_RemoveTexture(frame.barFrame.castbar.icon);
    CleanUI_RemoveTexture(frame.barFrame.castbar.border);
    CleanUI_RemoveTexture(frame.barFrame.castbar.shield);
    
    local trivial = frame.barFrame:GetScale() < 1;
    
    if (trivial) then
        frame.barFrame:SetScale(0.6);
    end
end 

function CleanUI_RemoveTexture(texture)
    texture:SetTexture(nil);
    texture:SetSize(1, 1);
    texture:SetTexCoord(0, 0, 0, 0);
    texture:ClearAllPoints();
    texture:SetPoint("CENTER", frame, "CENTER", 10000000, 0);
end

function CleanUI_UpdateNameplatesTextures(texture)
    for index, statusBar in pairs(namePlateStatusBars) do
        statusBar:SetStatusBarTexture(texture);
    end
end

function CleanUI_InitNameplateCombopoints(parent)
    local last = nil;
    for i=1, 5 do
        local combo = _G[parent:GetName()..i];
        if (combo == nil) then
            combo = parent:CreateTexture(parent:GetName()..i, "OVERLAY");
            combo:SetWidth(6);
            combo:SetHeight(2);
            combo:SetTexture(CleanUIData.CUI_STATUSBAR);

            if (i == 1) then
                combo:SetVertexColor(0.25, 0.75, 0.25, 1.0);
            elseif (i == 2) then
                combo:SetVertexColor(1.00, 1.00, 0.00, 1.0);
            elseif (i == 3) then
                combo:SetVertexColor(1.00, 0.75, 0.25, 1.0);
            elseif (i == 4) then
                combo:SetVertexColor(1.00, 0.50, 0.25, 1.0);
            elseif (i == 5) then
                combo:SetVertexColor(1.00, 0.10, 0.10, 1.0);
            end

            combo:Hide();
        end

        if (last) then
            combo:ClearAllPoints();
            combo:SetPoint("LEFT", last, "RIGHT", 1, 0);
        else
            combo:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0);
        end

        last = combo;
    end
end

function CleanUI_Nameplates_UpdateFrame(frame)
    -- focus
    if (CleanUIData.NAMEPLATES_HIGHLIGHTTARGET and UnitExists("target") and frame:GetAlpha() == 1.0) then
        frame.focus:Show();
    else
        frame.focus:Hide();
    end
        
    -- spells
    if (frame:GetAlpha() == 1.0 and UnitExists("target")) then
        local spell, rank, displayName, icon, startTime, endTime, isTradeSkill, castID, interrupt = UnitCastingInfo("target");
        if (interrupt) then
            frame.barFrame.castbar.cuiBar.interrupt:Show();
        else
            frame.barFrame.castbar.cuiBar.interrupt:Hide();
        end
    end

    -- combo points
    if (CleanUIData.NAMEPLATES_SHOWCOMBOPOINTS and UnitExists("target") and frame:GetAlpha() == 1.0) then
        frame.combopoints:Show();
        CleanUI_Nameplates_UpdateComboPoints(frame.combopoints);
    else
        frame.combopoints:Hide();
    end

    -- pvp flagged
    local playerFaction, targetFaction;
    if (frame:GetAlpha() == 1.0 and UnitIsPVP("target")) then
        playerFaction = UnitFactionGroup("player");
        targetFaction = UnitFactionGroup("target");

        if (playerFaction ~= targetFaction) then
            frame.pvp:Show();

            if (targetFaction == "Alliance") then
                frame.pvp:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance");
            elseif (targetFaction == "Horde") then
                frame.pvp:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde");
            else
                frame.pvp:Hide();
            end
        else
            frame.pvp:Hide();
        end
    else
        frame.pvp:Hide();
    end 
end

function CleanUI_Nameplates_UpdateComboPoints(parent)
    local combopoints = GetComboPoints("player", "target");
    for i=1, 5 do
        local combo = _G[parent:GetName()..i];

        if (i > combopoints) then
            combo:Hide();
        else
            combo:Show();
        end
    end
end

