function CleanUI_InitQuestFrame()

    CleanUI_QuestFrameHookEvents();
    CleanUI_RegisterQuestFrameEvents();

    CleanUI_CleanQuestMapFrame();
    CleanUI_CleanQuestLogDetailFrame();
    CleanUI_CleanQuestFrame();
    CleanUI_CleanQuestGreetingFrame();
    CleanUI_CleanLootFrame();
end

function CleanUI_UpdateWatchFrameAlpha()
    ObjectiveTrackerFrame:SetAlpha(CleanUIData.WATCHFRAME_ALPHA);
end

function CleanUI_RegisterQuestFrameEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:HookScript("OnEvent", CleanUI_QuestFrameOnEvent);
end

function CleanUI_QuestFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        --if ( addonName == "Blizzard_GarrisonUI" ) then
        --end
    end
end

function CleanUI_QuestFrameHookEvents()
    hooksecurefunc("QuestLogQuests_Update", CleanUI_QuestFrame_UpdateSize);
    hooksecurefunc("QuestMapFrame_ShowQuestDetails", CleanUI_QuestMapFrame_ShowQuestDetails);
end

function CleanUI_QuestFrame_UpdateSize()

    QuestScrollFrame:SetWidth(260);
    QuestScrollFrame:SetHeight(458);

    QuestScrollFrame.Background:SetHeight(466);
    QuestScrollFrame.Background:ClearAllPoints();
    QuestScrollFrame.Background:SetPoint("TOPLEFT", QuestScrollFrame, "TOPLEFT", 0, 4);
end

function CleanUI_CleanQuestMapFrame()
    CleanUI_RemoveTextures(QuestMapFrame);
    CleanUI_RemoveTextures(QuestMapFrame.DetailsFrame);

    CleanUI_RemoveTextures(QuestScrollFrame.StoryTooltip);
    CleanUI_SetBackdrop(QuestScrollFrame.StoryTooltip);

    local background = QuestMapFrame.DetailsFrame:CreateTexture("CUI_QuestLogPopupDetailFrameBackground", "BORDER");
    background:SetSize(10, 10);
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", QuestMapFrame.DetailsFrame, "TOPLEFT", 3, 10);
    background:SetPoint("BOTTOMRIGHT", QuestMapFrame.DetailsFrame.RewardsFrame, "TOPRIGHT", -22, -8);
    background:SetTexture("interface\\Addons\\CleanUI\\skins\\frame\\paper-background");

    CleanUI_QuestFrame_UpdateSize();

    QuestScrollFrameScrollBarTop:Hide();
    QuestScrollFrameScrollBarMiddle:Hide();
    QuestScrollFrameScrollBarBottom:Hide();

    CleanUI_ChangeButtonRed(QuestMapFrame.DetailsFrame.AbandonButton);
    CleanUI_ChangeButtonRed(QuestMapFrame.DetailsFrame.ShareButton);
    CleanUI_ChangeButtonRed(QuestMapFrame.DetailsFrame.TrackButton);
    CleanUI_ChangeButtonRed(QuestMapFrame.DetailsFrame.BackButton);

    CleanUI_ChangeButtonRed(QuestScrollFrame.ViewAll);
    
    local questDropdown = CreateFrame("Frame", "CUI_QuestDropdown", QuestMapFrame.DetailsFrame, "CUIDropDownMenuTemplate");
    questDropdown:SetSize(150, 22)
    questDropdown:SetPoint("TOPRIGHT", 40, 32);
    questDropdown.label:SetText("")
    CleanUI_ChangeDropDown(questDropdown);
    UIDropDownMenu_JustifyText(questDropdown, "LEFT");
    UIDropDownMenu_SetWidth(questDropdown, 150);
end

function CleanUI_CleanQuestLogDetailFrame()
    CleanUI_RemoveTextures(QuestLogPopupDetailFrame);
    CleanUI_RemoveTextures(QuestLogPopupDetailFrameScrollFrame);
    CleanUI_SetBackdrop(QuestLogPopupDetailFrame);

    CleanUI_ChangeCloseButton(QuestLogPopupDetailFrame);

    --CleanUI_CreateHeaderFrame(QuestLogPopupDetailFrame, "CUI_QuestLogPopupDetailFrameHeaderBorderframe", 250, QuestLogPopupDetailTitleText); 

    local background = QuestLogPopupDetailFrame:CreateTexture("CUI_QuestLogPopupDetailFrameBackground", "BORDER");
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", QuestLogPopupDetailFrame, "TOPLEFT", 3, -60);
    background:SetSize(308, 412);
    background:SetTexture("interface\\Addons\\CleanUI\\skins\\frame\\paper-background");

    CleanUI_RemoveTextures(QuestLogPopupDetailFrameInset);

    CleanUI_ChangeButtonRed(QuestLogPopupDetailFrameAbandonButton);
    CleanUI_ChangeButton(QuestLogPopupDetailFrameShareButton);
    CleanUI_ChangeButton(QuestLogPopupDetailFrameBackButton);
    CleanUI_ChangeButton(QuestLogPopupDetailFrameTrackButton);
end

function CleanUI_CleanQuestFrame()
    CleanUI_RemoveTextures(QuestFrame);
    CleanUI_SetBackdrop(QuestFrame);

    QuestFramePortrait:Hide();

    CleanUI_ChangeCloseButton(QuestFrame);

    CleanUI_CreateHeaderFrame(QuestFrame, "CUI_QuestFrameHeaderBorderframe", 250, QuestFrameNpcNameText);

    local background = QuestFrame:CreateTexture("CUI_QuestFrameBackground", "BORDER");
    background:ClearAllPoints();
    background:SetPoint("TOPLEFT", QuestFrame, "TOPLEFT", 3, -60);
    background:SetSize(308, 412);
    background:SetTexture("interface\\Addons\\CleanUI\\skins\\frame\\paper-background");

    CleanUI_RemoveTextures(QuestFrameInset);

    CleanUI_RemoveTextures(QuestDetailScrollFrame);
    CleanUI_RemoveTextures(QuestFrameDetailPanel);

    CleanUI_RemoveTextures(QuestProgressScrollFrame);
    CleanUI_RemoveTextures(QuestFrameProgressPanel);

    CleanUI_ChangeButton(QuestFrameAcceptButton);
    CleanUI_ChangeButton(QuestFrameDeclineButton);
    CleanUI_ChangeButton(QuestFrameCompleteButton);
    CleanUI_ChangeButtonRed(QuestFrameGoodbyeButton);

    CleanUI_RemoveTextures(QuestFrameRewardPanel);

    QuestRewardScrollFrameTop:Hide();
    QuestRewardScrollFrameMiddle:Hide();
    QuestRewardScrollFrameBottom:Hide();

    CleanUI_ChangeButton(QuestFrameCompleteQuestButton);
end

function CleanUI_CleanLootFrame()
    CleanUI_RemoveTextures(LootFrame);
    CleanUI_SetBackdrop(LootFrame);

    LootFramePortraitOverlay:Hide();

    CleanUI_ChangeCloseButton(LootFrame);

    LootFrameTitleText:Hide();
    local headerText = CleanUI_GetRegionOfTypeAtPos(LootFrame, "FontString", 0);
    CleanUI_CreateHeaderFrame(LootFrame, "CUI_LootFrameHeaderBorderframe", 115, headerText);

    CleanUI_RemoveTextures(LootFrameInset);
    CleanUI_RemoveTextures(LootButton1);
    CleanUI_RemoveTextures(LootButton2);
    CleanUI_RemoveTextures(LootButton3);
    CleanUI_RemoveTextures(LootButton4);
end

function CleanUI_CleanQuestGreetingFrame()
    CleanUI_RemoveTextures(QuestFrameGreetingPanel);

    QuestGreetingScrollFrameTop:Hide();
    QuestGreetingScrollFrameMiddle:Hide();
    QuestGreetingScrollFrameBottom:Hide();

    CleanUI_ChangeButton(QuestFrameGreetingGoodbyeButton);
end

function CleanUI_QuestMapFrame_ShowQuestDetails(questID)
    UIDropDownMenu_Initialize(CUI_QuestDropdown, CleanUI_QuestDropdown_Initialize);
end

function CleanUI_QuestDropdown_Initialize()
    local numEntries, numQuests = GetNumQuestLogEntries();
    
    local actQuestDetailID = QuestMapFrame.DetailsFrame.questID;
    local actQuestLogIndex = GetQuestLogIndexByID(actQuestDetailID);
    
    local actTitle = GetQuestLogTitle(actQuestLogIndex);
    UIDropDownMenu_SetText(CUI_QuestDropdown, actTitle);

    for questLogIndex = 1, numEntries do
        local title, level, suggestedGroup, isHeader, isCollapsed, isComplete, frequency, questID, startEvent, displayQuestID, isOnMap, hasLocalPOI, isTask, isStory = GetQuestLogTitle(questLogIndex);
        
        if ( isHeader ) then
            local eq_header = UIDropDownMenu_CreateInfo();
            eq_header.text = title;
            eq_header.isNotRadio = true;
            eq_header.notCheckable = 1;
            eq_header.isTitle = true;
            eq_header.func = function(frame) QuestMapFrame_ShowQuestDetails(questID) end;
            
            UIDropDownMenu_AddButton(eq_header);
        else 
            local info = UIDropDownMenu_CreateInfo();
            info.text = "    "..title;
            info.isNotRadio = true;
            info.notCheckable = 1;
            info.func = function(frame) QuestMapFrame_ShowQuestDetails(questID) end;
            
            if (questID == actQuestDetailID) then
                info.disabled = true
            end

            UIDropDownMenu_AddButton(info);
        end
    end
end
