function CleanUI_InitRaid()
    if (CleanUIData.CHANGE_RAID) then
        CleanUI_ChangeRaid();
        CleanUI_RaidHookEvents();
        CleanUI_RegisterRaidEvents();
    end
end

function CleanUI_RegisterRaidEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("GROUP_ROSTER_UPDATE");

    frame:HookScript("OnEvent", CleanUI_RaidOnEvent);
end

function CleanUI_RaidOnEvent(self, event, ...)
    if (event == "GROUP_ROSTER_UPDATE") then
        CleanUI_UpdateRaidFrame();
    end
end

function CleanUI_RaidHookEvents()
    if (CleanUIData.CHANGE_RAID) then
        hooksecurefunc("CompactRaidGroup_UpdateBorder", CleanUI_CompactRaidGroup_UpdateBorder);
        hooksecurefunc("CompactUnitFrame_UpdateSelectionHighlight", CleanUI_CompactUnitFrame_UpdateSelectionHighlight);
        hooksecurefunc("CompactUnitFrame_UpdateAggroHighlight", CleanUI_CompactUnitFrame_UpdateAggroHighlight);
    end
end

function CleanUI_ChangeRaid()
    CleanUI_UpdateRaidFrameManager();
--CleanUI_UpdateRaidFrame();

--CleanUI_ChangeButton(CompactRaidFrameManagerDisplayFrameLockedModeToggle);
--CleanUI_ChangeButton(CompactRaidFrameManagerDisplayFrameHiddenModeToggle);
end

function CleanUI_UpdateRaidFrameManager()
    local raidFrameManager = _G["CompactRaidFrameManager"];
    CleanUI_RemoveTextures(raidFrameManager);

    CleanUI_SetBackdrop(raidFrameManager);

    local displayFrame = _G["CompactRaidFrameManagerDisplayFrame"];
    CleanUI_RemoveTextures(displayFrame);

    local toggleButton = _G["CompactRaidFrameManagerToggleButton"];
    toggleButton:SetNormalTexture("Interface\\AddOns\\CleanUI\\skins\\ui\\toggle");
end

function CleanUI_UpdateRaidFrame()
    local raidBorderFrame = _G["CompactRaidFrameContainerBorderFrame"];
    CleanUI_RemoveTextures(raidBorderFrame);

    local raidGroup, raidgroupBorderFrame;
    for rg = 1, 8 do

        raidGroup = _G["CompactRaidGroup"..rg];

        if (raidGroup) then
            raidgroupBorderFrame = _G["CompactRaidGroup"..rg.."BorderFrame"];
            CleanUI_RemoveTextures(raidgroupBorderFrame);
            CleanUI_CompactRaidGroup_UpdateBorder(raidGroup);
        end
    end

    local party = _G["CompactPartyFrame"];
    if (party) then
        local partyBorderFrame = _G["CompactPartyFrameBorderFrame"];
        CleanUI_RemoveTextures(partyBorderFrame);
        CleanUI_CompactRaidGroup_UpdateBorder(party);
    end
end

function CleanUI_CompactRaidGroup_UpdateBorder(frame)
    local showBorder = frame.borderFrame:IsVisible();

    for uu = 1, 5 do
        local unitFrame = _G[frame:GetName().."Member"..uu];

        unitFrame.healthBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);
        unitFrame.powerBar:SetStatusBarTexture(CleanUIData.CUI_STATUSBAR);

        local borderframe = CleanUI_CreateRaidBorderFrame(unitFrame);

        if ( showBorder ) then
            borderframe:Show();
        else
            borderframe:Hide();
        end
    end
end

function CleanUI_CreateRaidBorderFrame(frame)

    -- status color
    frame.statusText:SetTextColor(0.7, 0.7, 0.7);

    -- orig. textures
    frame.background:SetTexture(nil);

    frame.horizDivider:SetTexture(nil);
    frame.horizTopBorder:SetTexture(nil);
    frame.horizBottomBorder:SetTexture(nil);
    frame.vertLeftBorder:SetTexture(nil);
    frame.vertRightBorder:SetTexture(nil);

    frame.healthBar.background:SetTexture(nil);
    frame.powerBar.background:SetTexture(nil);

    -- new border
    local borderframe = _G[frame:GetName().."Border"];
    if (not borderframe) then
        borderframe = CreateFrame("Frame", frame:GetName().."Border", frame);
    end

    CleanUI_SetBackdrop(borderframe, false);
    borderframe:SetPoint("TOPLEFT", -3, 1);
    borderframe:SetPoint("BOTTOMRIGHT", 2, -2);

    frame.borderframe = borderframe;

    return borderframe;
end

function CleanUI_CompactUnitFrame_UpdateSelectionHighlight(frame)
    local options = DefaultCompactUnitFrameSetupOptions;

    if (not frame.borderframe) then
        return;
    end

    if (not options.displayBorder) then
        frame.borderframe:Hide();
        return;
    end

    if ( not frame.optionTable.displaySelectionHighlight ) then
        frame.borderframe:Show();
        return;
    end

    if ( UnitIsUnit(frame.displayedUnit, "target") ) then
        frame.borderframe:Hide();
    else
        frame.borderframe:Show();
    end
end

function CleanUI_CompactUnitFrame_UpdateAggroHighlight(frame)
    local options = DefaultCompactUnitFrameSetupOptions;

    if (not frame.borderframe or not options.displayBorder) then
        return;
    end

    if (not options.displayBorder) then
        frame.borderframe:Hide();
        return;
    end

    if ( not frame.optionTable.displayAggroHighlight ) then
        frame.borderframe:Show();
        return;
    end

    local status = UnitThreatSituation(frame.displayedUnit);
    if ( status and status > 0 ) then
        frame.borderframe:Hide();
    else
        frame.borderframe:Show();
    end
end
