local queue = {};
local bag = {};

local tradeskill_professions = {
    ['01Tailoring'] = {spellid=3908, recipes=true},
    ['02Enchanting'] = {spellid=7411, recipes=true},
    ['04Smelting'] = {spellid=2656, recipes=true},
    ['05Blacksmithing'] = {spellid=3100, recipes=true},
    ['06Alchemy'] = {spellid=2259, recipes=true},
    ['07Leatherworking'] = {spellid=2108, recipes=true},
    ['08Engineering'] = {spellid=4036, recipes=true},
    ['09Jewelcrafting'] = {spellid=25229, recipes=true},
    ['10Inscription'] = {spellid=45357, recipes=true},
    ['11Runeforging'] = {spellid=53428, recipes=true},
    ['14Cooking'] = {spellid=2550, recipes=true},
    ['16First Aid'] = {spellid=3273, recipes=true},
};

local tradeskill_buttons = {};

local loaded = false;

local CREATE = "CREATE";
local BUY = "BUY";
local MISSING = "MISSING";
local MISSING_RESOURCE = "MISSING_RESOURCE";

local MASQUE = LibStub("Masque", true);
local buttongroup = nil;

function CleanUI_InitTradeSkillFrame()
    if (MASQUE) then
        buttongroup = MASQUE:Group("CleanUI", "TradeSkillFrame");
    end

    CleanUI_RegisterTradeSkillFrameEvents();
end

function CleanUI_RegisterTradeSkillFrameEvents()
    local frame = CreateFrame("Frame");

    frame:RegisterEvent("ADDON_LOADED");

    frame:RegisterEvent("TRADE_SKILL_UPDATE");
    frame:RegisterEvent("TRADE_SKILL_SHOW");
    frame:RegisterEvent("MERCHANT_SHOW");
    frame:RegisterEvent("MERCHANT_CLOSED");

    frame:HookScript("OnEvent", CleanUI_TradeSkillFrameOnEvent);
end

function CleanUI_TradeSkillHookEvents()
    hooksecurefunc("TradeSkillFrame_SetSelection", CleanUI_TradeSkillFrame_SetSelection);
    hooksecurefunc("TradeSkillFrame_Update", CleanUI_TradeSkillFrame_Update);
    hooksecurefunc("TradeSkillFrame_SetLinkName", CleanUI_TradeSkillFrame_SetLinkName);
    hooksecurefunc("TradeSkillFrame_Show", CleanUI_TradeSkillFrame_Show);
end

function CleanUI_InitTradeSkillButtons()
    local button;

    for name, data in pairs(tradeskill_professions) do
        local spellid = data.spellid;
        local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);

        -- Button in der Buttonleiste
        local button = CreateFrame("Button", "CUITRadeSkillButton"..name, TradeSkillFrame, "CleanUITradeSkillButtonTemplate");

        button:RegisterForClicks("AnyUp");
        button.rangeTimer = -1;

        button:RegisterEvent("PLAYER_TARGET_CHANGED");
        button:RegisterEvent("ACTIONBAR_UPDATE_USABLE");
        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");

        button:SetAttribute("type", "spell");
        button:SetAttribute("spell", spellname);
        button:SetAttribute("id", spellid);

        if (MASQUE) then
            buttongroup:AddButton(button);
        end

        _G[button:GetName().."Icon"]:SetTexture(texture);

        tradeskill_buttons[name] = {bar=button};
    end

    local count = 0;

    for name, data in pairs(tradeskill_professions) do
        local spellid = data.spellid;
        local spellname, rank, texture, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spellid);
        local usable, nomana = IsUsableSpell(spellname);

        local button = tradeskill_buttons[name].bar;

        if (usable) then
            button:ClearAllPoints();
            button:SetPoint("TOPRIGHT", -(20 + 40 * count), -35);

            _G[button:GetName().."Icon"]:SetTexture(GetSpellTexture(spellname));

            button:Show();
            count = count + 1;
        else
            button:Hide();
        end
    end
end

function CleanUI_TradeSkillFrameOnEvent(self, event, ...)
    if (event == "ADDON_LOADED") then
        local addonName = ...;
        if (addonName == "Blizzard_TradeSkillUI") then
            CleanUI_CleanTradeSkillFrame();
            CleanUI_TradeSkillHookEvents();
            CleanUI_InitTradeSkillButtons();
        end
    end
    if (event == "TRADE_SKILL_UPDATE") then
        CleanUI_TradeSkill_ReloadQueue();
    end
    if (event == "MERCHANT_SHOW" or event == "MERCHANT_CLOSED") then
        CleanUI_TradeSkill_ReloadQueue();
    end
    if (event == "TRADE_SKILL_SHOW") then
        CleanUI_TradeSkill_ReloadQueue();
    end
end

function CleanUI_TradeSkillFrame_SetLinkName()
    local linkedName = IsTradeSkillLinked();
    TradeSkillFrameTitleText:ClearAllPoints();
    if ( linkedName ) then
        local linkedNameWidth = TradeSkillFrameDummyString:GetWidth();
        if linkedNameWidth > TRADE_SKILL_LINKED_NAME_WIDTH then
            linkedNameWidth = TRADE_SKILL_LINKED_NAME_WIDTH;
        end
        TradeSkillFrameTitleText:SetPoint("TOP", -linkedNameWidth/2,  -6);
    else
        TradeSkillLinkNameButton:Hide();
        TradeSkillFrameTitleText:SetPoint("TOP", 0,  -6);
    end
end

function CleanUI_TradeSkillFrame_Show()
    TradeSkillFramePortrait:Hide();
end

function CleanUI_CleanTradeSkillFrame()

    TradeSkillFrame:SetSize(682, 447);

    TradeSkillListScrollFrame:SetSize(304, 330);
    TradeSkillListScrollFrame:ClearAllPoints();
    TradeSkillListScrollFrame:SetPoint("TOPLEFT", TradeSkillFrame, "TOPLEFT", 20, -87);

    CleanUI_RemoveTextures(TradeSkillFrame);
    CleanUI_SetBackdrop(TradeSkillFrame);

    CleanUI_RemoveTextures(TradeSkillFrameInset);
    CleanUI_RemoveTextures(TradeSkillExpandButtonFrame);

    TradeSkillFramePortrait:Hide();

    CleanUI_RemoveTextures(TradeSkillListScrollFrame);
    CleanUI_RemoveTextures(TradeSkillDetailScrollFrame);
    CleanUI_RemoveTextures(TradeSkillDetailScrollChildFrame);

    CleanUI_CreateHeaderFrame(TradeSkillFrame, "CUI_TradeSkillFrameHeaderBorderframe", 250, TradeSkillFrameTitleText);
    CleanUI_ChangeCloseButton(TradeSkillFrame);

    TradeSkillFrameSearchBox:ClearAllPoints();
    TradeSkillFrameSearchBox:SetPoint("TOPLEFT", TradeSkillRankFrame, "BOTTOMLEFT", 5, -5);

    CleanUI_ChangeEditBox(TradeSkillFrameSearchBox);
    CleanUI_ChangeEditBox(TradeSkillInputBox);

    TradeSkillRankFrame:SetHeight(20);
    TradeSkillRankFrameBorder:SetHeight(36);
    TradeSkillRankFrame:SetPoint("TOPLEFT", 66, -25);

    TradeSkillDetailScrollFrame:SetSize(297, 174);
    TradeSkillDetailScrollFrame:ClearAllPoints();
    TradeSkillDetailScrollFrame:SetPoint("TOPRIGHT", TradeSkillFrame, "TOPRIGHT", -36, -89);

    TradeSkillHorizontalBarLeft:ClearAllPoints();
    TradeSkillHorizontalBarLeft:SetPoint("TOPLEFT", 347, -261);
    TradeSkillHorizontalBarLeft:SetPoint("RIGHT", -80, -53);

    TradeSkillLinkButton:SetSize(18, 18);
    TradeSkillLinkButton:SetNormalTexture("Interface\\Chatframe\\UI-ChatWhisperIcon");
    TradeSkillLinkButton:SetPushedTexture("Interface\\Chatframe\\UI-ChatWhisperIcon");

    TradeSkillFilterButtonTopLeft:Hide();
    TradeSkillFilterButtonTopMiddle:Hide();
    TradeSkillFilterButtonTopRight:Hide();
    TradeSkillFilterButtonMiddleLeft:Hide();
    TradeSkillFilterButtonMiddleMiddle:Hide();
    TradeSkillFilterButtonMiddleRight:Hide();
    TradeSkillFilterButtonBottomLeft:Hide();
    TradeSkillFilterButtonBottomMiddle:Hide();
    TradeSkillFilterButtonBottomRight:Hide();

    TRADE_SKILLS_DISPLAYED = 20;
    local button;

    for i=9, TRADE_SKILLS_DISPLAYED, 1 do
        button = CreateFrame("Button", "TradeSkillSkill"..i, TradeSkillFrame, "TradeSkillSkillButtonTemplate");
        button:SetPoint("TOPLEFT", _G["TradeSkillSkill"..(i-1)], "BOTTOMLEFT", 0, 0);
    end

    TradeSkillInputBox:ClearAllPoints();
    TradeSkillInputBox:SetPoint("RIGHT", TradeSkillIncrementButton, "LEFT", 0, 0);
    TradeSkillDecrementButton:ClearAllPoints();
    TradeSkillDecrementButton:SetPoint("RIGHT", TradeSkillInputBox, "LEFT", -4, 0);

    local queueButton = CreateFrame("Button", "CUI_TRADESKILL_QUEUEBUTTON", TradeSkillFrame, "MagicButtonTemplate");
    queueButton:SetText(CUI_TRADESKILL_QUEUE);
    queueButton:SetSize(85, 22);
    queueButton:ClearAllPoints();
    queueButton:SetPoint("RIGHT", TradeSkillDecrementButton, "LEFT", -4, 0);
    queueButton:HookScript("OnClick", CleanUI_TradeSkillCreateQueue);
    CleanUI_ChangeButton(queueButton);

    local clearButton = CreateFrame("Button", "CUI_TRADESKILL_CLEARBUTTON", TradeSkillFrame, "MagicButtonTemplate");
    clearButton:SetText(CUI_TRADESKILL_CLEAR);
    clearButton:SetSize(85, 22);
    clearButton:ClearAllPoints();
    clearButton:SetPoint("RIGHT", queueButton, "LEFT", 0, 0);
    clearButton:HookScript("OnClick", CleanUI_TradeSkillClearQueue);
    CleanUI_ChangeButton(clearButton);

    local queueScrollFrame = CreateFrame("ScrollFrame", "CUITradeSkillQueueScrollFrame", TradeSkillFrame, "CUITradeSkillQueueScrollFrameTemplate");
    queueScrollFrame:SetSize(295, 142);
    queueScrollFrame:ClearAllPoints();
    queueScrollFrame:SetPoint("BOTTOMRIGHT", TradeSkillFrame, "BOTTOMRIGHT", -32, 33);
    CleanUI_RemoveTextures(queueScrollFrame);

    HybridScrollFrame_OnLoad(queueScrollFrame);
    queueScrollFrame.update = CleanUI_TradeSkillUpdateQueue;
    HybridScrollFrame_CreateButtons(queueScrollFrame, "CUITradeSkillQueueEntryTemplate");
    loaded = true;

    CleanUI_ChangeButton(TradeSkillViewGuildCraftersButton);
    TradeSkillFrameTabardEmblem:ClearAllPoints();
    TradeSkillFrameTabardEmblem:SetPoint("TOPLEFT", TradeSkillFrame, "TOPLEFT", 5, 0);

    CleanUI_ChangeButton(TradeSkillCreateButton);
    CleanUI_ChangeButton(TradeSkillCreateAllButton);
    CleanUI_ChangeButtonRed(TradeSkillCancelButton);

    CleanUI_TradeSkillUpdateQueue();
end

function CleanUI_TradeSkillFrame_Update()
    local rankBar;
    local min, max, value;
    for i=1, TRADE_SKILLS_DISPLAYED, 1 do
        rankBar = _G["TradeSkillSkill"..i.."SubSkillRankBar"];

        if (rankBar) then
            min, max = rankBar:GetMinMaxValues();
            value = rankBar:GetValue();

            if (value == max) then
                rankBar:Hide();
            end
        end
    end
end

function CleanUI_TradeSkillFrame_SetSelection(skillIndex)
    local link = GetTradeSkillItemLink(skillIndex);

    local r, g, b;

    if (link) then
        local itemName, itemLink, itemRarity = GetItemInfo(link);

        if (itemName) then
            local texture = _G["CUI_TradeSkillSkillIconBorder"];

            if (not texture) then
                texture = TradeSkillSkillIcon:CreateTexture("CUI_TradeSkillSkillIconBorder", "ARTWORK");
                texture:SetPoint("TOPLEFT", -18, 17);
                texture:SetPoint("BOTTOMRIGHT", 18, -19);

                texture:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
                texture:Show();
            end

            local icon = TradeSkillSkillIcon:GetNormalTexture();
            if (icon) then
                icon:ClearAllPoints();
                icon:SetPoint("TOPLEFT", TradeSkillSkillIcon, "TOPLEFT", 0, 0);
                icon:SetPoint("BOTTOMRIGHT", TradeSkillSkillIcon, "BOTTOMRIGHT", 0, 0);
                icon:SetTexCoord(0.1, 0.9, 0.1, 0.9);
            end

            r, g, b = 0.5, 0.5, 0.5;

            if (itemRarity) then
                r, g, b = GetItemQualityColor(itemRarity);
            end

            texture:SetVertexColor(r, g, b);
        end

        local reagent, reagentTexture;
        local reagentName, reagentLink, reagentRarity;

        for i=1, 6 do
            reagent = _G["TradeSkillReagent"..i];
            _G["TradeSkillReagent"..i.."NameFrame"]:Hide();

            if (reagent:IsVisible()) then
                reagentTexture = _G["CUI_TradeSkillReagent"..i.."IconBorder"];

                if (not reagentTexture) then
                    reagentTexture = reagent:CreateTexture("CUI_TradeSkillReagent"..i.."IconBorder", "ARTWORK");
                    reagentTexture:ClearAllPoints();
                    reagentTexture:SetPoint("TOPLEFT", reagent, "TOPLEFT", -13, 13);
                    reagentTexture:SetPoint("BOTTOMRIGHT", reagent, "BOTTOMRIGHT", -94, -14);

                    reagentTexture:SetTexture("interface\\AddOns\\CleanUI\\skins\\bag\\item_border");
                    reagentTexture:Show();
                end

                reagentName, reagentLink, reagentRarity = GetItemInfo(GetTradeSkillReagentItemLink(skillIndex, i));

                r, g, b = 0.5, 0.5, 0.5;

                if (reagentRarity) then
                    r, g, b = GetItemQualityColor(reagentRarity);
                end

                reagentTexture:SetVertexColor(r, g, b);
            end
        end
    end
end

function CleanUITradeSkillQueue_OnEnter(self)
    if (self and self.data and self.data.itemLink) then
        GameTooltip_SetDefaultAnchor(GameTooltip, self);
        GameTooltip:SetOwner(self, "ANCHOR_RIGHT");

        GameTooltip:SetHyperlink(self.data.itemLink);
    end
end

function CleanUITradeSkillQueue_OnLeave(self)
    GameTooltip:FadeOut();
end

function CleanUITradeSkillQueue_OnClick(self, button, down)
    PlaySound("ACTIONBARBUTTONDOWN");
    local data = self.data;

    if (data) then
        local index = CleanUI_GetSkillIndexForName(data.reagent);

        if (index) then
            SelectTradeSkill(index);
            TradeSkillFrame_Show();
        end
    end
end

function CleanUITradeSkillQueue_ButtonClicked(button)
    if (button.type == CREATE) then
        local index = CleanUI_GetSkillIndexForName(button.data.reagent);
        DoTradeSkill(index, button.data.count);

        local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine();
        local savedQueue = CleanUITradeSkillQueue[tradeskillName];

        if (savedQueue) then
            for i, data in ipairs(savedQueue) do
                if (data.name == button.data.reagent) then
                    data.count = data.count - button.data.count;

                    if (data.count <= 0) then
                        tremove(CleanUITradeSkillQueue[tradeskillName], i);
                    end

                    break;
                end
            end
        end
    elseif (button.type == BUY) then
        BuyMerchantItem(button.index, button.buyQuantity);
    end

    CleanUI_TradeSkill_ReloadQueue();
end

function CleanUI_GetSkillIndexForName(name)
    for skillIndex = 1, GetNumTradeSkills(), 1 do
        skillName, skillType, numAvailable = CleanUI_GetTradeSkillInfo(skillIndex);

        if (skillName == name) then
            return skillIndex;
        end
    end

    return nil;
end

function CleanUI_TradeSkill_ReloadQueue()
    local help = {};

    local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine();
    local savedQueue = CleanUITradeSkillQueue[tradeskillName];

    if (savedQueue) then
        for i, data in ipairs(savedQueue) do
            tinsert(help, data);
        end
    end

    CleanUI_TradeSkillClearQueue();

    for i, data in ipairs(help) do
        if (data and data.count > 0) then
            local skillIndex = CleanUI_GetSkillIndexForName(data.name);

            if (skillIndex) then
                CleanUI_TradeSkillCreateQueueForIndex(skillIndex, data.count);
            end
        end
    end
end

function CleanUI_TradeSkill_AddToQueue(skillType, reagent, count, itemLink, type, initial)
    if (count <= 0) then
        return;
    end

    local added = nil;

    for index, data in ipairs(queue) do
        if (skillType == data.skillType and
            reagent == data.reagent and
            itemLink == data.itemLink and
            type == data.type) then
            data.count = data.count + count;
            added = 1;
            break;
        end
    end

    if (not added) then
        local data = {skillType = skillType, reagent = reagent, count = count, itemLink = itemLink, type = type };

        if (initial == 1) then
            tinsert(queue, data);
        else
            tinsert(queue, 1, data);
        end
    end

    if (type == CREATE) then
        local skillIndex = CleanUI_GetSkillIndexForName(reagent);
        local numReagents = GetTradeSkillNumReagents(skillIndex);

        local reagentName, reagentTexture, reagentCount, playerReagentCount;
        for reagentIndex = 1, numReagents, 1 do
            reagentName, reagentTexture, reagentCount, playerReagentCount = CleanUI_GetTradeSkillReagentInfo(skillIndex, reagentIndex);
            if (bag[reagentName]) then
                bag[reagentName] = bag[reagentName] - (count * reagentCount);
            end
        end
    end

    CleanUI_TradeSkillUpdateQueue();
end

function CleanUI_TradeSkillUpdateQueue()
    if (not loaded) then
        return;
    end

    local scrollFrame = CUITradeSkillQueueScrollFrame;

    local buttons = scrollFrame.buttons;

    local numEntries = #queue;
    local numButtons = #buttons;

    local scrollOffset = HybridScrollFrame_GetOffset(scrollFrame);
    local buttonHeight = buttons[1]:GetHeight();
    local displayedHeight = 0;

    local button, queueIndex, data, font;
    for i=1, numButtons do
        queueIndex = i + scrollOffset;
        button = buttons[i];

        if ( queueIndex <= numEntries ) then
            data = queue[queueIndex];
            button:Show();

            button.count:SetText("|cffffffff"..data.count.."|r");
            button:SetText(data.reagent);

            if (TradeSkillTypeColor[data.skillType]) then
                font = TradeSkillTypeColor[data.skillType].font;
                button:SetNormalFontObject(font);
            else
                button:SetNormalFontObject(GameFontNormal);
            end

            button.createButton:SetScale(0.75);

            if (data.type == CREATE) then
                button.createButton:SetText(CREATE_PROFESSION);
                button.createButton:Show();
                button.createButton:Enable();
                button.createButton.type = CREATE;
            elseif (data.type == MISSING_RESOURCE) then
                button.createButton:SetText(CREATE_PROFESSION);
                button.createButton:Show();
                button.createButton:Disable();
            else
                local index, buyQuantity, price, extendedCost = CleanUI_TradeSkillCheckMerchant(data.reagent, data.count);

                if (index) then
                    button.createButton:Show();
                    button.createButton.type = BUY;
                    button.createButton.index = index;
                    button.createButton.buyQuantity = buyQuantity;
                    button.createButton.price = price;

                    if (price > 0) then
                        button.createButton:SetText(GetCoinTextureString(price * buyQuantity));
                    elseif ( extendedCost and (price > 0) ) then
                        button.createButton:SetText(price * buyQuantity.." units");
                    else
                        button.createButton:SetText(CUI_TRADESKILL_BUY);
                    end

                    button.createButton:Enable();
                else
                    button.createButton:Hide();
                    button.createButton:Disable();
                end
            end

            button:SetNormalTexture("");
            button:SetHighlightTexture("");
            button.data = data;
            button.createButton.data = data;
            button.queueIndex = queueIndex;
        else
            button:Hide();
            button.data = nil;
        end

        displayedHeight = displayedHeight + buttonHeight;
    end

    HybridScrollFrame_Update(scrollFrame, numEntries * buttonHeight, displayedHeight);
end

function CleanUI_TradeSkillCheckMerchant(reagent, count)
    if (not MerchantFrame:IsShown()) then
        return nil;
    end

    local name, texture, price, quantity, numAvailable, isUsable, extendedCost;
    local buyQuantity;
    for index = 1, GetMerchantNumItems(), 1 do
        name, texture, price, quantity, numAvailable, isUsable, extendedCost = GetMerchantItemInfo(index);
        if (name == reagent) then
            buyQuantity = count;

            if (numAvailable > 0 and buyQuantity > numAvailable) then
                buyQuantity = numAvailable;
            end

            return index, buyQuantity, price, extendedCost;
        end
    end

    return nil;
end

function CleanUI_TradeSkillClearQueue()
    queue = {};
    bag = {};

    local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine();
    CleanUITradeSkillQueue[tradeskillName] = {};

    CleanUI_TradeSkillUpdateQueue();
end

function CleanUI_TradeSkillCreateQueue()
    local skillIndex = GetTradeSkillSelectionIndex();

    -- how much
    local repeatCount = 1;

    if (TradeSkillInputBox) then
        repeatCount = TradeSkillInputBox:GetNumber();
    end

    CleanUI_TradeSkillCreateQueueForIndex(skillIndex, repeatCount);
end

function CleanUI_TradeSkillCreateQueueForIndex(skillIndex, repeatCount)
    CleanUI_TradeSkillScanBags();

    local skillName, skillType, numAvailable, isExpanded, altVerb, numSkillUps = CleanUI_GetTradeSkillInfo(skillIndex);
    local itemLink = GetTradeSkillItemLink(skillIndex);

    -- save
    local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine();
    tinsert(CleanUITradeSkillQueue[tradeskillName], {name=skillName, count=repeatCount});

    -- create
    local skillIndexForReagent, skillTypeForReagent, numAvailableForReagent = CleanUI_TradeSkill_CheckRecipeForReagent(skillName);
    local rest;

    if (numAvailableForReagent >= repeatCount) then
        CleanUI_TradeSkill_AddToQueue(skillType, skillName, repeatCount, itemLink, CREATE, 1);
    else
        rest = repeatCount - numAvailableForReagent;

        if (numAvailableForReagent > 0) then
            CleanUI_TradeSkill_AddToQueue(skillType, skillName, numAvailableForReagent, itemLink, CREATE, 1);
        end

        CleanUI_TradeSkill_AddToQueue(skillType, skillName, rest, itemLink, MISSING_RESOURCE, 1);
        CleanUI_TradeSkill_CkeckIndex(skillIndex, rest);
    end
end

function CleanUI_TradeSkill_CkeckIndex(skillIndex, count)
    local skillName, skillType, numAvailable, isExpanded, altVerb, numSkillUps = CleanUI_GetTradeSkillInfo(skillIndex);

    local numReagents = GetTradeSkillNumReagents(skillIndex);
    local reagentName, reagentTexture, reagentCount, playerReagentCount, reagentItemLink;
    local skillIndexForReagent, skillTypeForReagent, numAvailableForReagent, notAvailableForReagent, required;

    for reagentIndex = 1, numReagents, 1 do
        reagentName, reagentTexture, reagentCount, playerReagentCount = CleanUI_GetTradeSkillReagentInfo(skillIndex, reagentIndex);
        reagentItemLink = GetTradeSkillReagentItemLink(skillIndex, reagentIndex);
        reagentCount = reagentCount * count;
        required = reagentCount - playerReagentCount;

        if (reagentName) then

            skillIndexForReagent, skillTypeForReagent, numAvailableForReagent = CleanUI_TradeSkill_CheckRecipeForReagent(reagentName);

            if (skillIndexForReagent) then
                if (numAvailableForReagent > 0) then
                    CleanUI_TradeSkill_AddToQueue(skillTypeForReagent, reagentName, min(required, numAvailableForReagent), reagentItemLink, CREATE);
                end

                notAvailableForReagent = required - numAvailableForReagent;
                if (notAvailableForReagent > 0) then
                    CleanUI_TradeSkill_AddToQueue(skillTypeForReagent, reagentName, notAvailableForReagent, reagentItemLink, MISSING_RESOURCE);
                    CleanUI_TradeSkill_CkeckIndex(skillIndexForReagent, notAvailableForReagent);
                end
            else
                notAvailableForReagent = required;
                CleanUI_TradeSkill_AddToQueue(nil, reagentName, notAvailableForReagent, reagentItemLink, MISSING);
            end
        end
    end;
end

function CleanUI_GetTradeSkillReagentInfo(skillIndex, reagentIndex)
    local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(skillIndex, reagentIndex);
    return reagentName, reagentTexture, reagentCount, bag[reagentName] or 0;
end

function CleanUI_GetTradeSkillInfo(skillIndex)
    local skillName, skillType, numAvailable, isExpanded, altVerb, numSkillUps = GetTradeSkillInfo(skillIndex);

    local numReagents = GetTradeSkillNumReagents(skillIndex);

    local reagentName, reagentTexture, reagentCount, playerReagentCount;
    local foundInBags, help;
    for reagentIndex = 1, numReagents, 1 do
        reagentName, reagentTexture, reagentCount, playerReagentCount = CleanUI_GetTradeSkillReagentInfo(skillIndex, reagentIndex);
        foundInBags = bag[reagentName];

        if (foundInBags) then
            help = math.floor(foundInBags / reagentCount);
            if (help < numAvailable) then
                numAvailable = help;
            end
        else
            numAvailable = 0;
        end
    end

    return skillName, skillType, numAvailable, isExpanded, altVerb, numSkillUps;
end

function CleanUI_TradeSkill_CheckRecipeForReagent(reagentName)
    local skillName, skillType, numAvailable;
    for skillIndex = 1, GetNumTradeSkills(), 1 do
        skillName, skillType, numAvailable = CleanUI_GetTradeSkillInfo(skillIndex);

        if (skillName == reagentName) then
            return skillIndex, skillType, numAvailable;
        end
    end

    return nil, nil, nil;
end

function xxxCleanUI_TradeSkillScanBags()
    local containerid, container, numberOfSlots, itemLink, itemName;

    for id = 0, 4, 1 do
        containerid = "ContainerFrame"..(id + 1);
        container = getglobal(containerid);

        if (container) then
            numberOfSlots = GetContainerNumSlots(id);
            for slot=1, numberOfSlots, 1 do
                itemLink = GetContainerItemLink(id, slot);

                if (itemLink) then
                    itemName = GetItemInfo(itemLink);
                    if (not bag[itemName] and itemName) then
                        bag[itemName] = GetItemCount(itemLink, true);
                    end
                end
            end
        end
    end
end

function CleanUI_TradeSkillScanBags()
    local bagdata = CleanUIDS_GetBagDataSortedByType(UnitGUID("player"));
    
    local itemName;

    for itemType, typeData in pairs(bagdata) do
        for itemLink, data in pairs(typeData) do            
            itemName = GetItemInfo(itemLink);
            bag[itemName] = data.itemCount;
        end
    end
end

function CleanUI_TradeSkillButtonMouseOver(button)
    local spellid = button:GetAttribute("id");

    GameTooltip_SetDefaultAnchor(GameTooltip, button);
    GameTooltip:SetSpellByID(spellid);

    GameTooltip:AppendText("");
end

function CleanUI_TradeSkillButtonMouseOut(button)
    GameTooltip:FadeOut();
    selectedButton = nil;
end
